#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#     ||          ____  _ __
#  +------+      / __ )(_) /_______________ _____  ___
#  | 0xBC |     / __  / / __/ ___/ ___/ __ `/_  / / _ \
#  +------+    / /_/ / / /_/ /__/ /  / /_/ / / /_/  __/
#   ||  ||    /_____/_/\__/\___/_/   \__,_/ /___/\___/
#
#  Copyright (C) 2011-2023 Bitcraze AB
#
#  Crazyflie Nano Quadcopter Client
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  You should have received a copy of the GNU General Public License along with
#  this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""
Dialogue used to select and configure an inputdevice. This includes mapping
buttons and axis to match controls for the Crazyflie.
"""
import logging

import cfclient
from PyQt6.QtCore import Qt
from PyQt6.QtCore import QThread
from PyQt6.QtCore import QTimer
from PyQt6.QtCore import pyqtSignal
from PyQt6.QtWidgets import QMessageBox
from cfclient.utils.config_manager import ConfigManager
from PyQt6 import QtWidgets
from PyQt6 import uic

__author__ = 'Bitcraze AB'
__all__ = ['InputConfigDialogue']

logger = logging.getLogger(__name__)

(inputconfig_widget_class, connect_widget_base_class) = (
    uic.loadUiType(cfclient.module_path + '/ui/dialogs/inputconfigdialogue.ui')
)


class InputConfigDialogue(QtWidgets.QWidget, inputconfig_widget_class):

    def __init__(self, joystickReader, *args):
        super(InputConfigDialogue, self).__init__(*args)
        self.setupUi(self)
        self._input = joystickReader

        self._input_device_reader = DeviceReader(self._input)
        self._input_device_reader.start()

        self._input_device_reader.raw_axis_data_signal.connect(
            self._detect_axis)
        self._input_device_reader.raw_button_data_signal.connect(
            self._detect_button)
        self._input_device_reader.mapped_values_signal.connect(
            self._update_mapped_values)

        self.cancelButton.clicked.connect(self.close)
        self.saveButton.clicked.connect(self._save_config)

        self.detectPitch.clicked.connect(
            lambda: self._axis_detect(
                "pitch", "Pitch axis",
                "Center the pitch axis then do max %s pitch",
                ["forward", "backward"]))
        self.detectRoll.clicked.connect(
            lambda: self._axis_detect(
                "roll", "Roll axis",
                "Center the roll axis and then do max %s roll",
                ["right", "left"]))
        self.detectYaw.clicked.connect(
            lambda: self._axis_detect(
                "yaw", "Yaw axis",
                "Center the yaw axis and then do max %s yaw",
                ["right", "left"]))
        self.detectThrust.clicked.connect(
            lambda: self._axis_detect(
                "thrust", "Thrust axis",
                "Center the thrust axis, and then do max thrust"))
        self.detectPitchPos.clicked.connect(
            lambda: self._button_detect(
                "pitchPos", "Pitch Cal Positive",
                "Press the button for Pitch postive calibration"))
        self.detectPitchNeg.clicked.connect(
            lambda: self._button_detect(
                "pitchNeg", "Pitch Cal Negative",
                "Press the button for Pitch negative calibration"))
        self.detectRollPos.clicked.connect(
            lambda: self._button_detect(
                "rollPos", "Roll Cal Positive",
                "Press the button for Roll positive calibration"))
        self.detectRollNeg.clicked.connect(
            lambda: self._button_detect(
                "rollNeg", "Roll Cal Negative",
                "Press the button for Roll negative calibration"))
        self.detectKillswitch.clicked.connect(
            lambda: self._button_detect(
                "estop", "Disarm/Kill",
                "Press the button to disarm/kill (will disable motors)"))
        self.detectArm.clicked.connect(
            lambda: self._button_detect(
                "arm", "Arm system",
                "Press the button to arm the system (will enable motor ouput)"))
        self.detectAlt1.clicked.connect(
            lambda: self._button_detect(
                "alt1", "Alternative function 1",
                "The alternative function 1 that will do a callback"))
        self.detectAlt2.clicked.connect(
            lambda: self._button_detect(
                "alt2", "Alternative function 2",
                "The alternative function 2 that will do a callback"))
        self.detectExitapp.clicked.connect(
            lambda: self._button_detect(
                "exitapp", "Exit application",
                "Press the button for exiting the application"))
        self._detect_assisted_control.clicked.connect(
            lambda: self._button_detect(
                "assistedControl", "Assisted control",
                "Press the button for assisted control mode activation "
                "(releasing returns to manual mode)"))
        self.detectMuxswitch.clicked.connect(
            lambda: self._button_detect(
                "muxswitch", "Mux Switch",
                "Press the button for mux switching"))

        self.configButton.clicked.connect(self._start_configuration)
        self.loadButton.clicked.connect(self._load_config_from_file)
        self.deleteButton.clicked.connect(self._delete_configuration)

        self._popup = None
        self._combined_button = None
        self._detection_buttons = [
            self.detectPitch, self.detectRoll,
            self.detectYaw, self.detectThrust,
            self.detectPitchPos, self.detectPitchNeg,
            self.detectRollPos, self.detectRollNeg,
            self.detectKillswitch, self.detectArm,
            self.detectExitapp, self._detect_assisted_control,
            self.detectAlt1, self.detectAlt2,
            self.detectMuxswitch]

        self._button_to_detect = ""
        self._axis_to_detect = ""
        self.combinedDetection = 0
        self._prev_combined_id = None

        self._maxed_axis = []
        self._mined_axis = []

        self._buttonindicators = {}
        self._axisindicators = {}
        self._reset_mapping()

        for d in self._input.available_devices():
            if d.supports_mapping:
                self.inputDeviceSelector.addItem(d.name, d.id)

        if len(self._input.available_devices()) > 0:
            self.configButton.setEnabled(True)

        self._map = {}
        self._saved_open_device = None

    @staticmethod
    def _scale(max_value, value):
        return (value / max_value) * 100

    def _reset_mapping(self):
        self._buttonindicators = {
            "pitchPos": self.pitchPos,
            "pitchNeg": self.pitchNeg,
            "rollPos": self.rollPos,
            "rollNeg": self.rollNeg,
            "estop": self.killswitch,
            "arm": self.arm,
            "alt1": self.alt1,
            "alt2": self.alt2,
            "exitapp": self.exitapp,
            "assistedControl": self._assisted_control,
            "muxswitch": self.muxswitch,
        }

        self._axisindicators = {
            "pitch": self.pitchAxisValue,
            "roll": self.rollAxisValue,
            "yaw": self.yawAxisValue,
            "thrust": self.thrustAxisValue,
        }

    def _cancel_config_popup(self, button):
        self._axis_to_detect = ""
        self._button_to_detect = ""

    def _show_config_popup(self, caption, message, directions=[]):
        self._maxed_axis = []
        self._mined_axis = []
        self._popup = QMessageBox()
        self._popup.directions = directions
        self._combined_button = QtWidgets.QPushButton('Combined Axis ' +
                                                      'Detection')
        self.cancelButton = QtWidgets.QPushButton('Cancel')
        self._popup.addButton(self.cancelButton, QMessageBox.ButtonRole.DestructiveRole)
        self._popup.setWindowTitle(caption)
        self._popup.setWindowFlags(Qt.WindowType.Dialog | Qt.WindowType.MSWindowsFixedSizeDialogHint)
        if len(directions) > 1:
            self._popup.originalMessage = message
            message = self._popup.originalMessage % directions[0]
            self._combined_button.setCheckable(True)
            self._combined_button.blockSignals(True)
            self._popup.addButton(self._combined_button,
                                  QMessageBox.ButtonRole.ActionRole)
        self._popup.setText(message)
        self._popup.show()

    def _start_configuration(self):
        self._input.enableRawReading(
            str(self.inputDeviceSelector.currentText()))
        self._input_device_reader.start_reading()
        self._populate_config_dropdown()
        self.profileCombo.setEnabled(True)
        for b in self._detection_buttons:
            b.setEnabled(True)

    def _detect_axis(self, data):
        if (len(self._axis_to_detect) > 0):
            if (self._combined_button and self._combined_button.isChecked() and
                    self.combinedDetection == 0):
                self._combined_button.setDisabled(True)
                self.combinedDetection = 1
            for a in data:
                # Axis must go low and high before it's accepted as selected
                # otherwise maxed out axis (like gyro/acc) in some controllers
                # will always be selected. Not enforcing negative values makes
                # it possible to detect split axis (like bumpers on PS3
                # controller)
                if a not in self._maxed_axis and abs(data[a]) > 0.8:
                    self._maxed_axis.append(a)
                if a not in self._mined_axis and abs(data[a]) < 0.1:
                    self._mined_axis.append(a)
                if a in self._maxed_axis and a in self._mined_axis and len(
                        self._axis_to_detect) > 0:
                    if self.combinedDetection == 0:
                        if data[a] >= 0:
                            self._map_axis(self._axis_to_detect, a, 1.0)
                        else:
                            self._map_axis(self._axis_to_detect, a, -1.0)
                        self._axis_to_detect = ""
                        self._check_and_enable_saving()
                        if self._popup is not None:
                            self.cancelButton.click()
                    elif self.combinedDetection == 2:  # finished detection
                        # not the same axis again ...
                        if self._prev_combined_id != a:
                            self._map_axis(self._axis_to_detect, a, -1.0)
                            self._axis_to_detect = ""
                            self._check_and_enable_saving()
                            if (self._popup is not None):
                                self.cancelButton.click()
                            self.combinedDetection = 0
                    elif self.combinedDetection == 1:
                        self._map_axis(self._axis_to_detect, a, 1.0)
                        self._prev_combined_id = a
                        self.combinedDetection = 2
                        message = (self._popup.originalMessage %
                                   self._popup.directions[1])
                        self._popup.setText(message)

    def _update_mapped_values(self, mapped_data):
        for v in mapped_data.get_all_indicators():
            if v in self._buttonindicators:
                if mapped_data.get(v):
                    self._buttonindicators[v].setChecked(True)
                else:
                    self._buttonindicators[v].setChecked(False)
            if v in self._axisindicators:
                # The sliders used are set to 0-100 and the values from the
                # input-layer is scaled according to the max settings in
                # the input-layer. So scale the value and place 0 in the middle
                scaled_value = mapped_data.get(v)
                if v == "thrust":
                    scaled_value = InputConfigDialogue._scale(
                        self._input.max_thrust, scaled_value
                    )
                if v == "roll" or v == "pitch":
                    scaled_value = InputConfigDialogue._scale(
                        self._input.max_rp_angle, scaled_value
                    )
                if v == "yaw":
                    scaled_value = InputConfigDialogue._scale(
                        self._input.max_yaw_rate, scaled_value
                    )
                self._axisindicators[v].setValue(int(scaled_value))

    def _map_axis(self, function, key_id, scale):
        self._map["Input.AXIS-{}".format(key_id)] = {}
        self._map["Input.AXIS-{}".format(key_id)]["id"] = key_id
        self._map["Input.AXIS-{}".format(key_id)]["key"] = function
        self._map["Input.AXIS-{}".format(key_id)]["scale"] = scale
        self._map["Input.AXIS-{}".format(key_id)]["offset"] = 0.0
        self._map["Input.AXIS-{}".format(key_id)]["type"] = "Input.AXIS"
        self._input.set_raw_input_map(self._map)

    def _map_button(self, function, key_id):
        # Duplicate buttons are not allowed, remove if there's already one
        # mapped
        prev_button = None
        for m in self._map:
            if "key" in self._map[m] and self._map[m]["key"] == function:
                prev_button = m
        if prev_button:
            del self._map[prev_button]

        self._map["Input.BUTTON-{}".format(key_id)] = {}
        self._map["Input.BUTTON-{}".format(key_id)]["id"] = key_id
        self._map["Input.BUTTON-{}".format(key_id)]["key"] = function
        self._map["Input.BUTTON-{}".format(key_id)]["scale"] = 1.0
        self._map["Input.BUTTON-{}".format(key_id)]["type"] = "Input.BUTTON"
        self._input.set_raw_input_map(self._map)

    def _detect_button(self, data):
        if len(self._button_to_detect) > 0:
            for b in data:
                if data[b] > 0:
                    self._map_button(self._button_to_detect, b)
                    self._button_to_detect = ""
                    self._check_and_enable_saving()
                    if self._popup is not None:
                        self._popup.close()

    def _check_and_enable_saving(self):
        needed_funcs = ["thrust", "yaw", "roll", "pitch"]

        for m in self._map:
            if self._map[m]["key"] in needed_funcs:
                needed_funcs.remove(self._map[m]["key"])

        if len(needed_funcs) == 0:
            self.saveButton.setEnabled(True)

    def _populate_config_dropdown(self):
        configs = ConfigManager().get_list_of_configs()
        if len(configs):
            self.loadButton.setEnabled(True)
        for c in configs:
            self.profileCombo.addItem(c)

    def _axis_detect(self, varname, caption, message, directions=[]):
        self._axis_to_detect = varname
        self._show_config_popup(caption, message, directions)

    def _button_detect(self, varname, caption, message):
        self._button_to_detect = varname
        self._show_config_popup(caption, message)

    def _show_error(self, caption, message):
        QMessageBox.critical(self, caption, message)

    def _load_config_from_file(self):
        loaded_map = ConfigManager().get_config(
            self.profileCombo.currentText())
        if loaded_map:
            self._input.set_raw_input_map(loaded_map)
            self._map = loaded_map
        else:
            logger.warning("Could not load configfile [%s]",
                           self.profileCombo.currentText())
            self._show_error("Could not load config",
                             "Could not load config [%s]" %
                             self.profileCombo.currentText())
        self._check_and_enable_saving()

    def _delete_configuration(self):
        logger.warning("deleteConfig not implemented")

    def _save_config(self):
        config_name = str(self.profileCombo.currentText())
        ConfigManager().save_config(self._map, config_name)
        self.close()

    def showEvent(self, event):
        """Called when dialog is opened"""
        # self._saved_open_device = self._input.get_device_name()
        # self._input.stop_input()
        self._input.pause_input()

    def closeEvent(self, event):
        """Called when dialog is closed"""
        self._input.stop_raw_reading()
        self._input_device_reader.stop_reading()
        # self._input.start_input(self._saved_open_device)
        self._input.resume_input()


class DeviceReader(QThread):
    """Used for polling data from the Input layer during configuration"""
    raw_axis_data_signal = pyqtSignal(object)
    raw_button_data_signal = pyqtSignal(object)
    mapped_values_signal = pyqtSignal(object)

    def __init__(self, input):
        QThread.__init__(self)

        self._input = input
        self._read_timer = QTimer()
        self._read_timer.setInterval(25)

        self._read_timer.timeout.connect(self._read_input)

    def stop_reading(self):
        """Stop polling data"""
        self._read_timer.stop()

    def start_reading(self):
        """Start polling data"""
        self._read_timer.start()

    def _read_input(self):
        [rawaxis, rawbuttons, mapped_values] = self._input.read_raw_values()
        self.raw_axis_data_signal.emit(rawaxis)
        self.raw_button_data_signal.emit(rawbuttons)
        self.mapped_values_signal.emit(mapped_values)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#     ||          ____  _ __
#  +------+      / __ )(_) /_______________ _____  ___
#  | 0xBC |     / __  / / __/ ___/ ___/ __ `/_  / / _ \
#  +------+    / /_/ / / /_/ /__/ /  / /_/ / / /_/  __/
#   ||  ||    /_____/_/\__/\___/_/   \__,_/ /___/\___/
#
#  Copyright (C) 2011-2013 Bitcraze AB
#
#  Crazyflie Nano Quadcopter Client
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA  02110-1301, USA.

# Crazy Loader bootloader utility
# Can reset bootload and reset back the bootloader

import sys

import cflib.crtp
from cflib.bootloader import Bootloader, Target
from cflib.bootloader.boottypes import BootVersion


def main():
    # Initialise the CRTP link driver
    try:
        cflib.crtp.init_drivers()
    except Exception as e:
        print("Error: {}".format(str(e)))
        sys.exit(-1)

    # Set the default parameters
    clink = None
    action = "info"
    boot = "cold"
    filename = None  # type: Optional[str]
    targets = None  # type: Optional[List[Target]]
    bl = None  # type: Optional[Bootloader]

    exit_result = 0

    if len(sys.argv) < 2:
        print()
        print("==============================")
        print(" CrazyLoader Flash Utility")
        print("==============================")
        print()
        print(" Usage:", sys.argv[0], "[CRTP options] <action> [parameters]")
        print()
        print("The CRTP options are described above")
        print()
        print("Crazyload option:")
        print("   info                    : Print the info of the bootloader "
              "and quit.")
        print("                             Will let the target in bootloader "
              "mode")
        print("   reset                   : Reset the device in firmware mode")
        print("   flash <file> [targets]  : flash the <img> binary file from "
              "the first")
        print("                             possible  page in flash and reset "
              "to firmware")
        print("                             mode.")
        sys.exit(0)

    # Analyse the command line parameters
    sys.argv = sys.argv[1:]
    argv = []

    i = 0
    while i < len(sys.argv):
        if sys.argv[i] == "--cold-boot" or sys.argv[i] == "-c":
            boot = "cold"
        elif sys.argv[i] == "--warm-boot" or sys.argv[i] == "-w":
            boot = "reset"
            i += 1
            clink = sys.argv[i]
        else:
            argv += [sys.argv[i]]
        i += 1
    sys.argv = argv

    # Analyse the command
    if len(sys.argv) < 1:
        action = "info"
    elif sys.argv[0] == "info":
        action = "info"
    elif sys.argv[0] == "reset":
        action = "reset"
    elif sys.argv[0] == "flash":
        if len(sys.argv) < 2:
            print("The flash action require a file name.")
            sys.exit(-1)
        action = "flash"
        filename = sys.argv[1]
        targets = []  # Dict[Target]
        for t in sys.argv[2:]:
            if t.startswith("deck-"):
                [deck, target, type] = t.split("-")
                targets.append(Target("deck", target, type, [], []))
            else:
                [target, type] = t.split("-")
                targets.append(Target("cf2", target, type, [], []))
    else:
        print("Action", sys.argv[0], "unknown!")
        sys.exit(-1)

    try:
        # Initialise the bootloader lib
        bl = Bootloader(clink)

        warm_boot = (boot == "reset")
        if warm_boot:
            print("Reset to bootloader mode ...")
            sys.stdout.flush()
        else:  # The connection is done by a cold boot ...
            print("Restart the Crazyflie you want to bootload in the next"),
            print(" 10 seconds ..."),

            sys.stdout.flush()

        ######################################
        # Doing something (hopefully) useful
        ######################################

        if action == "info":
            def print_info(version: int, connected_targets: [Target]):
                print("Connected to bootloader on {} (version=0x{:X})".format(
                    BootVersion.to_ver_string(version),
                    version
                    )
                )
                for target in connected_targets:
                    print(target)

            # flash_full called with no filename will not flash, just call
            # our info callback
            bl.flash_full(None, None, warm_boot, None, print_info)
        elif action == "flash" and filename:
            is_target_required = not filename.endswith('.zip')
            if (is_target_required and not targets):
                print("The flash action with a non .zip file requires a target")
                sys.exit(-1)
            try:
                bl.flash_full(None, filename, warm_boot, targets)
            except Exception as e:
                print("Failed to flash: {}".format(e))
                exit_result = -1
        elif action == "reset":
            bl.reset_to_firmware()
        else:
            None
    except Exception as e:
        import traceback

        traceback.print_exc(file=sys.stdout)
        print(e)
        exit_result = -1

    finally:
        #########################
        # Closing the connection
        #########################
        if bl:
            bl.close()

    sys.exit(exit_result)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#     ||          ____  _ __
#  +------+      / __ )(_) /_______________ _____  ___
#  | 0xBC |     / __  / / __/ ___/ ___/ __ `/_  / / _ \
#  +------+    / /_/ / / /_/ /__/ /  / /_/ / / /_/  __/
#   ||  ||    /_____/_/\__/\___/_/   \__,_/ /___/\___/
#
#  Copyright (C) 2015 Bitcraze AB
#
#  Crazyflie Nano Quadcopter Client
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
#  USA.

"""
Give access to the LED driver memory via ZMQ.
"""

from cflib.crazyflie.mem import MemoryElement
from cfclient.utils.config import Config

import logging
from threading import Thread, Lock

ZMQ_PULL_PORT = 1024 + 190
logger = logging.getLogger(__name__)

enabled = False
try:
    import zmq

    enabled = True
except Exception as e:
    logger.warning("Not enabling ZMQ LED driver access,"
                   "import failed ({})".format(e))

if not Config().get("enable_zmq_led"):
    enabled = False
    logger.info("ZMQ led disabled in config file")


class _PullReader(Thread):
    """Blocking thread for reading from ZMQ socket"""

    def __init__(self, receiver, callback, *args):
        """Initialize"""
        super(_PullReader, self).__init__(*args)
        self._receiver = receiver
        self._cb = callback
        self.daemon = True
        self.lock = Lock()

    def run(self):
        while True:
            # self.lock.acquire()
            self._cb(self._receiver.recv_json())


class ZMQLEDDriver:
    """Used for reading data from input devices using the PyGame API."""

    def __init__(self, crazyflie):

        if enabled:
            self._cf = crazyflie
            context = zmq.Context()
            self._receiver = context.socket(zmq.PULL)
            self._bind_addr = "tcp://*:{}".format(ZMQ_PULL_PORT)
            # If the port is already bound an exception will be thrown
            # and caught in the initialization of the readers and handled.
            self._receiver.bind(self._bind_addr)
            logger.info("Biding ZMQ for LED driver"
                        "at {}".format(self._bind_addr))
            self._receiver_thread = _PullReader(self._receiver,
                                                self._cmd_callback)

    def start(self):
        if enabled:
            self._receiver_thread.start()

    def _cmd_callback(self, data):
        """Called when new data arrives via ZMQ"""
        if len(self._cf.mem.get_mems(MemoryElement.TYPE_DRIVER_LED)) > 0:
            logger.info("Updating memory")
            memory = self._cf.mem.get_mems(MemoryElement.TYPE_DRIVER_LED)[0]
            for i_led in range(len(data["rgbleds"])):
                memory.leds[i_led].set(data["rgbleds"][i_led][0],
                                       data["rgbleds"][i_led][1],
                                       data["rgbleds"][i_led][2])
            memory.write_data(self._write_cb)

    def _write_cb(self, mem, addr):
        return



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#     ||          ____  _ __
#  +------+      / __ )(_) /_______________ _____  ___
#  | 0xBC |     / __  / / __/ ___/ ___/ __ `/_  / / _ \
#  +------+    / /_/ / / /_/ /__/ /  / /_/ / / /_/  __/
#   ||  ||    /_____/_/\__/\___/_/   \__,_/ /___/\___/
#
#  Copyright (C) 2013-2023 Bitcraze AB
#
#  Crazyflie Nano Quadcopter Client
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA  02110-1301, USA.

"""
This tab shows all log blocks that are registered and can be used to start the
logging and also to write the logging data to file.
"""

from PyQt6 import uic
from PyQt6.QtCore import Qt, pyqtSignal

import cfclient
from cfclient.ui.tab_toolbox import TabToolbox

import logging

from PyQt6.QtWidgets import QApplication, QStyledItemDelegate
from PyQt6.QtWidgets import QAbstractItemView, QStyleOptionButton, QStyle
from PyQt6.QtCore import QAbstractItemModel, QModelIndex

from cfclient.utils.logdatawriter import LogWriter

__author__ = 'Bitcraze AB'
__all__ = ['LogBlockTab']

logblock_tab_class = uic.loadUiType(cfclient.module_path + "/ui/tabs/logBlockTab.ui")[0]

logger = logging.getLogger(__name__)


class LogBlockChildItem(object):
    """Class that acts as a child in the tree and represents one variable in
    a log block"""

    def __init__(self, parent, name):
        """Initialize the node"""
        self.parent = parent
        self.name = name

    def child_count(self):
        """Return the number of children this node has"""
        return 0


class LogBlockItem(object):
    """Class that acts as a parent in the tree view and represents a complete
    log block"""

    def __init__(self, block, model, connected_ts):
        """Initialize the parent node"""
        super(LogBlockItem, self).__init__()
        self._block = block
        self.parent = None
        self.children = []
        self.name = block.name
        self.id = block.id
        self.period = block.period_in_ms
        self._model = model
        self._log_file_writer = LogWriter(block, connected_ts)

        self._block.started_cb.add_callback(self._set_started)
        self._block.added_cb.add_callback(self._set_added)
        self._block.error_cb.add_callback(self._log_error)

        self._var_list = ""

        for b in block.variables:
            self.children.append(LogBlockChildItem(self, b.name))
            self._var_list += "%s/" % b.name
        self._var_list = self._var_list[:-1]

        self._block_started = False
        self._doing_transaction = False

    def _log_error(self, logconfig, msg):
        """
        Callback when there's an error starting the block in the Crazyflie
        """
        # Do nothing here, a pop-up will notify the user that the
        # starting failed
        self._doing_transaction = False

    def _set_started(self, conf, started):
        """Callback when a block has been started in the Crazyflie"""
        logger.debug("%s started: %s", self.name, started)
        if started:
            self._block_started = True
        else:
            self._block_started = False
        self._doing_transaction = False
        self._model.refresh()

    def logging_started(self):
        """Return True if the block has been started, otherwise False"""
        return self._block_started

    def writing_to_file(self):
        """Return True if the block is being logged to file, otherwise False"""
        return self._log_file_writer.writing()

    def start_writing_to_file(self):
        """Start logging to file for this block"""
        self._log_file_writer.start()

    def stop_writing_to_file(self):
        """Stop logging to file for this block"""
        self._log_file_writer.stop()

    def start(self):
        """Start the logging of this block"""
        self._doing_transaction = True
        self._block.start()

    def stop(self):
        """Stop the logging of this block"""
        self._doing_transaction = True
        self._block.delete()

    def doing_transaction(self):
        """Return True if a block is being added or started, False when it's
        been added/started/failed"""
        return self._doing_transaction

    def _set_added(self, conf, started):
        """Callback when a block has been added to the Crazyflie"""
        logger.debug("%s added: %s", self.name, started)

    def var_list(self):
        """Return a string containing all the variable names of the children"""
        return self._var_list

    def child_count(self):
        """Return the number of children this node has"""
        return len(self.children)

    def get_child(self, index):
        return self.children[index]


class LogBlockModel(QAbstractItemModel):

    def __init__(self, view, parent=None):
        super(LogBlockModel, self).__init__(parent)
        self._nodes = []
        self._column_headers = ['Id', 'Name', 'Period (ms)', 'Start',
                                'Write to file', 'Contents']
        self._view = view
        self._nodes_written_to_file = []

    def add_block(self, block, connected_ts):
        self._nodes.append(LogBlockItem(block, self, connected_ts))
        self.layoutChanged.emit()
        self._nodes.sort(key=lambda conf: conf.name.lower())

    def refresh(self):
        """Force a refresh of the view though the model"""
        self.layoutChanged.emit()

    def clicked(self, index):
        """
        Callback when a cell has been clicked (mouse down/up on same cell)
        """
        node = index.internalPointer()
        if not node.parent and index.column() == 3:
            if node.logging_started():
                node.stop()
            else:
                node.start()
        if not node.parent and index.column() == 4:
            if node.writing_to_file():
                node.stop_writing_to_file()
            else:
                node.start_writing_to_file()
        self.layoutChanged.emit()

    def parent(self, index):
        """Re-implemented method to get the parent of the given index"""
        if not index.isValid():
            return QModelIndex()

        node = index.internalPointer()
        if node.parent is None:
            return QModelIndex()
        else:
            return self.createIndex(self._nodes.index(node.parent), 0,
                                    node.parent)

    def remove_block(self, block):
        """Remove a block from the view"""
        raise NotImplementedError()

    def columnCount(self, parent):
        """Re-implemented method to get the number of columns"""
        return len(self._column_headers)

    def headerData(self, section, orientation, role):
        """Re-implemented method to get the headers"""
        if role == Qt.ItemDataRole.DisplayRole:
            return self._column_headers[section]

    def rowCount(self, parent):
        """Re-implemented method to get the number of rows for a given index"""
        parent_item = parent.internalPointer()
        if parent.isValid():
            parent_item = parent.internalPointer()
            return parent_item.child_count()
        else:
            return len(self._nodes)

    def index(self, row, column, parent):
        """Re-implemented method to get the index for a specified
        row/column/parent combination"""
        if not self._nodes:
            return QModelIndex()
        node = parent.internalPointer()
        if not node:
            index = self.createIndex(row, column, self._nodes[row])
            self._nodes[row].index = index
            return index
        else:
            return self.createIndex(row, column, node.get_child(row))

    def data(self, index, role):
        """Re-implemented method to get the data for a given index and role"""
        node = index.internalPointer()
        parent = node.parent
        if parent:
            if role == Qt.ItemDataRole.DisplayRole and index.column() == 5:
                return node.name
        elif not parent and role == Qt.ItemDataRole.DisplayRole and index.column() == 5:
            return node.var_list()
        elif not parent and role == Qt.ItemDataRole.DisplayRole:
            if index.column() == 0:
                return node.id
            if index.column() == 1:
                return node.name
            if index.column() == 2:
                return str(node.period)
        if role == Qt.ItemDataRole.TextAlignmentRole and \
                (index.column() == 4 or index.column() == 3):
            return Qt.AlignmentFlag.AlignHCenter | Qt.AlignmentFlag.AlignVCenter

        return None

    def reset(self):
        """Reset the model"""
        # Stop the logging to file
        for node in self._nodes:
            if node.writing_to_file():
                node.stop_writing_to_file()
        self._nodes = []
        self.layoutChanged.emit()


class CheckboxDelegate(QStyledItemDelegate):
    """Custom delegate for rending checkboxes in the table"""

    def paint(self, painter, option, index):
        """Re-implemented paint method"""
        item = index.internalPointer()
        col = index.column()
        if not item.parent and (col == 3 or col == 4):
            s = QStyleOptionButton()
            checkbox_rect = QApplication.style().subElementRect(QStyle.SubElement.SE_CheckBoxIndicator, option)
            s.rect = option.rect
            center_offset = int(s.rect.width() / 2 - checkbox_rect.width() / 2)
            s.rect.adjust(center_offset, 0, 0, 0)

            if col == 3:
                if not item.doing_transaction():
                    s.state = QStyle.StateFlag.State_Enabled
                if item.logging_started():
                    s.state |= QStyle.StateFlag.State_On

            if col == 4:
                s.state = QStyle.StateFlag.State_Enabled
                if item.writing_to_file():
                    s.state |= QStyle.StateFlag.State_On

            self._paint_checkbox(s, painter)
        else:
            super(CheckboxDelegate, self).paint(painter, option, index)

    def _paint_checkbox(self, style, painter):
        QApplication.style().drawControl(QStyle.ControlElement.CE_CheckBox, style, painter)

    def _paint_checkbox_osx_workaround(self, style, painter):
        painter.save()
        painter.translate(style.rect.topLeft())
        QApplication.style().drawControl(QStyle.ControlElement.CE_CheckBox, style, painter)
        painter.restore()


class LogBlockTab(TabToolbox, logblock_tab_class):
    """
    Used to show debug-information about logblock status.
    """

    _blocks_updated_signal = pyqtSignal(bool)
    _disconnected_signal = pyqtSignal(str)

    def __init__(self, helper):
        """Initialize the tab"""
        super(LogBlockTab, self).__init__(helper, 'Log Blocks')
        self.setupUi(self)

        self._helper.cf.log.block_added_cb.add_callback(self._block_added)
        self._disconnected_signal.connect(self._disconnected)
        self._helper.cf.disconnected.add_callback(
            self._disconnected_signal.emit)

        self._model = LogBlockModel(self._block_tree)
        self._block_tree.setModel(self._model)
        self._block_tree.clicked.connect(self._model.clicked)
        self._block_tree.setItemDelegate(CheckboxDelegate())
        self._block_tree.setSelectionMode(QAbstractItemView.SelectionMode.NoSelection)

    def _block_added(self, block):
        """Callback from logging layer when a new block is added"""
        self._model.add_block(block, self._helper.cf.connected_ts)

    def _disconnected(self, link_uri):
        """Callback when the Crazyflie is disconnected"""
        self._model.beginResetModel()
        self._model.reset()
        self._model.endResetModel()



# -*- coding: utf-8 -*-
#
#     ||          ____  _ __
#  +------+      / __ )(_) /_______________ _____  ___
#  | 0xBC |     / __  / / __/ ___/ ___/ __ `/_  / / _ \
#  +------+    / /_/ / / /_/ /__/ /  / /_/ / / /_/  __/
#   ||  ||    /_____/_/\__/\___/_/   \__,_/ /___/\___/
#
#  Copyright (C) 2022-2023 Bitcraze AB
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA  02110-1301, USA.

"""
Wizard to estimate the geometry of the lighthouse base stations.
Used in the lighthouse tab from the manage geometry dialog
"""

from __future__ import annotations

import cfclient

from cflib.crazyflie import Crazyflie
from cflib.crazyflie.mem.lighthouse_memory import LighthouseBsGeometry
from cflib.localization.lighthouse_sweep_angle_reader import LighthouseSweepAngleAverageReader
from cflib.localization.lighthouse_sweep_angle_reader import LighthouseSweepAngleReader
from cflib.localization.lighthouse_bs_vector import LighthouseBsVectors
from cflib.localization.lighthouse_initial_estimator import LighthouseInitialEstimator
from cflib.localization.lighthouse_sample_matcher import LighthouseSampleMatcher
from cflib.localization.lighthouse_system_aligner import LighthouseSystemAligner
from cflib.localization.lighthouse_geometry_solver import LighthouseGeometrySolver
from cflib.localization.lighthouse_system_scaler import LighthouseSystemScaler
from cflib.localization.lighthouse_types import Pose, LhDeck4SensorPositions, LhMeasurement, LhCfPoseSample

from PyQt6 import QtCore, QtWidgets, QtGui
import time


REFERENCE_DIST = 1.0
ITERATION_MAX_NR = 2
DEFAULT_RECORD_TIME = 20
TIMEOUT_TIME = 2000
STRING_PAD_TOTAL = 6
WINDOW_STARTING_WIDTH = 780
WINDOW_STARTING_HEIGHT = 720
SPACER_LABEL_HEIGHT = 27
PICTURE_WIDTH = 640


class LighthouseBasestationGeometryWizard(QtWidgets.QWizard):
    def __init__(self, cf, ready_cb, parent=None, *args):
        super(LighthouseBasestationGeometryWizard, self).__init__(parent)
        self.cf = cf
        self.ready_cb = ready_cb
        self.wizard_opened_first_time = True
        self.reset()

        self.button(QtWidgets.QWizard.WizardButton.FinishButton).clicked.connect(self._finish_button_clicked_callback)

    def _finish_button_clicked_callback(self):
        self.ready_cb(self.get_geometry_page.get_geometry())

    def reset(self):
        self.setWindowFlags(self.windowFlags() & ~QtCore.Qt.WindowType.WindowContextHelpButtonHint)
        self.setWindowFlags(self.windowFlags() & ~QtCore.Qt.WindowType.WindowCloseButtonHint)

        if not self.wizard_opened_first_time:
            self.removePage(0)
            self.removePage(1)
            self.removePage(2)
            self.removePage(3)
            self.removePage(4)
            del self.get_origin_page, self.get_xaxis_page, self.get_xyplane_page
            del self.get_xyzspace_page, self.get_geometry_page
        else:
            self.wizard_opened_first_time = False

        self.get_origin_page = RecordOriginSamplePage(self.cf, self)
        self.get_xaxis_page = RecordXAxisSamplePage(self.cf, self)
        self.get_xyplane_page = RecordXYPlaneSamplesPage(self.cf, self)
        self.get_xyzspace_page = RecordXYZSpaceSamplesPage(self.cf, self)
        self.get_geometry_page = EstimateBSGeometryPage(
            self.cf, self.get_origin_page, self.get_xaxis_page, self.get_xyplane_page, self.get_xyzspace_page, self)

        self.addPage(self.get_origin_page)
        self.addPage(self.get_xaxis_page)
        self.addPage(self.get_xyplane_page)
        self.addPage(self.get_xyzspace_page)
        self.addPage(self.get_geometry_page)

        self.setWindowTitle("Lighthouse Base Station Geometry Wizard")
        self.resize(WINDOW_STARTING_WIDTH, WINDOW_STARTING_HEIGHT)


class LighthouseBasestationGeometryWizardBasePage(QtWidgets.QWizardPage):

    def __init__(self, cf: Crazyflie, show_add_measurements=False, parent=None):
        super(LighthouseBasestationGeometryWizardBasePage, self).__init__(parent)
        self.show_add_measurements = show_add_measurements
        self.cf = cf
        self.layout = QtWidgets.QVBoxLayout()

        self.explanation_picture = QtWidgets.QLabel()
        self.explanation_picture.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.layout.addWidget(self.explanation_picture)

        self.explanation_text = QtWidgets.QLabel()
        self.explanation_text.setText(' ')
        self.explanation_text.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.layout.addWidget(self.explanation_text)

        self.layout.addStretch()

        self.extra_layout_field()

        self.status_text = QtWidgets.QLabel()
        self.status_text.setFont(QtGui.QFont('Courier New', 10))
        self.status_text.setText(self.str_pad(''))
        self.status_text.setFrameStyle(QtWidgets.QFrame.Shape.Panel | QtWidgets.QFrame.Shadow.Plain)
        self.layout.addWidget(self.status_text)

        self.start_action_button = QtWidgets.QPushButton("Start Measurement")
        self.start_action_button.clicked.connect(self._action_btn_clicked)
        action_button_h_box = QtWidgets.QHBoxLayout()
        action_button_h_box.addStretch()
        action_button_h_box.addWidget(self.start_action_button)
        action_button_h_box.addStretch()
        self.layout.addLayout(action_button_h_box)
        self.setLayout(self.layout)
        self.is_done = False
        self.too_few_bs = False
        self.timeout_timer = QtCore.QTimer()
        self.timeout_timer.timeout.connect(self._timeout_cb)
        self.reader = LighthouseSweepAngleAverageReader(self.cf, self._ready_cb)
        self.recorded_angle_result = None
        self.recorded_angles_result: list[LhCfPoseSample] = []

    def isComplete(self):
        return self.is_done and (self.too_few_bs is not True)

    def extra_layout_field(self):
        self.spacer = QtWidgets.QLabel()
        self.spacer.setText(' ')
        self.spacer.setFixedSize(50, SPACER_LABEL_HEIGHT)
        self.layout.addWidget(self.spacer)

    def _action_btn_clicked(self):
        self.is_done = False
        self.reader.start_angle_collection()
        self.timeout_timer.start(TIMEOUT_TIME)
        self.status_text.setText(self.str_pad('Collecting sweep angles...'))
        self.start_action_button.setDisabled(True)

    def _timeout_cb(self):
        if self.is_done is not True:
            self.status_text.setText(self.str_pad('No sweep angles recorded! \n' +
                                     'Make sure that the lighthouse base stations are turned on!'))
            self.reader.stop_angle_collection()
            self.start_action_button.setText("Restart Measurement")
            self.start_action_button.setDisabled(False)
        elif self.too_few_bs:
            self.timeout_timer.stop()

    def _ready_cb(self, averages):
        print(self.show_add_measurements)
        recorded_angles = averages
        angles_calibrated = {}
        for bs_id, data in recorded_angles.items():
            angles_calibrated[bs_id] = data[1]
        self.recorded_angle_result = LhCfPoseSample(angles_calibrated=angles_calibrated)
        self.visible_basestations = ', '.join(map(lambda x: str(x + 1), recorded_angles.keys()))
        amount_of_basestations = len(recorded_angles.keys())

        if amount_of_basestations < 2:
            self.status_text.setText(self.str_pad('Recording Done!' +
                                                  f' Visible Base stations: {self.visible_basestations}\n' +
                                                  'Received too few base stations,' +
                                                  'we need at least two. Please try again!'))
            self.too_few_bs = True
            self.is_done = True
            if self.show_add_measurements and len(self.recorded_angles_result) > 0:
                self.too_few_bs = False
                self.completeChanged.emit()
            self.start_action_button.setText("Restart Measurement")
            self.start_action_button.setDisabled(False)
        else:
            self.too_few_bs = False
            status_text_string = f'Recording Done! Visible Base stations: {self.visible_basestations}\n'
            if self.show_add_measurements:
                self.recorded_angles_result.append(self.get_sample())
                status_text_string += f'Total measurements added: {len(self.recorded_angles_result)}\n'
            self.status_text.setText(self.str_pad(status_text_string))
            self.is_done = True
            self.completeChanged.emit()

            if self.show_add_measurements:
                self.start_action_button.setText("Add more measurements")
                self.start_action_button.setDisabled(False)
            else:
                self.start_action_button.setText("Restart Measurement")
            self.start_action_button.setDisabled(False)

    def get_sample(self):
        return self.recorded_angle_result

    def str_pad(self, string_msg):
        new_string_msg = string_msg

        if string_msg.count('\n') < STRING_PAD_TOTAL:
            for i in range(STRING_PAD_TOTAL-string_msg.count('\n')):
                new_string_msg += '\n'

        return new_string_msg


class RecordOriginSamplePage(LighthouseBasestationGeometryWizardBasePage):
    def __init__(self, cf: Crazyflie, parent=None):
        super(RecordOriginSamplePage, self).__init__(cf)
        self.explanation_text.setText(
            'Step 1. Put the Crazyflie where you want the origin of your coordinate system.\n')
        pixmap = QtGui.QPixmap(cfclient.module_path + "/ui/wizards/bslh_1.png")
        pixmap = pixmap.scaledToWidth(PICTURE_WIDTH)
        self.explanation_picture.setPixmap(pixmap)


class RecordXAxisSamplePage(LighthouseBasestationGeometryWizardBasePage):
    def __init__(self, cf: Crazyflie, parent=None):
        super(RecordXAxisSamplePage, self).__init__(cf)
        self.explanation_text.setText('Step 2. Put the Crazyflie on the positive X-axis,' +
                                      f'  exactly {REFERENCE_DIST} meters from the origin.\n' +
                                      'This will be used to define the X-axis as well as scaling of the system.')
        pixmap = QtGui.QPixmap(cfclient.module_path + "/ui/wizards/bslh_2.png")
        pixmap = pixmap.scaledToWidth(PICTURE_WIDTH)
        self.explanation_picture.setPixmap(pixmap)


class RecordXYPlaneSamplesPage(LighthouseBasestationGeometryWizardBasePage):
    def __init__(self, cf: Crazyflie, parent=None):
        super(RecordXYPlaneSamplesPage, self).__init__(cf, show_add_measurements=True)
        self.explanation_text.setText('Step 3. Put the Crazyflie somewhere in the XY-plane, but not on the X-axis.\n' +
                                      'This position is used to map the the XY-plane to the floor.\n' +
                                      'You can sample multiple positions to get a more precise definition.')
        pixmap = QtGui.QPixmap(cfclient.module_path + "/ui/wizards/bslh_3.png")
        pixmap = pixmap.scaledToWidth(PICTURE_WIDTH)
        self.explanation_picture.setPixmap(pixmap)

    def get_samples(self):
        return self.recorded_angles_result


class RecordXYZSpaceSamplesPage(LighthouseBasestationGeometryWizardBasePage):
    def __init__(self, cf: Crazyflie, parent=None):
        super(RecordXYZSpaceSamplesPage, self).__init__(cf)
        self.explanation_text.setText('Step 4. Move the Crazyflie around, try to cover all of the flying space,\n' +
                                      'make sure all the base stations are received.\n' +
                                      'Avoid moving too fast, you can increase the record time if needed.\n')
        pixmap = QtGui.QPixmap(cfclient.module_path + "/ui/wizards/bslh_4.png")
        pixmap = pixmap.scaledToWidth(PICTURE_WIDTH)
        self.explanation_picture.setPixmap(pixmap)

        self.record_timer = QtCore.QTimer()
        self.record_timer.timeout.connect(self._record_timer_cb)
        self.record_time_total = DEFAULT_RECORD_TIME
        self.record_time_current = 0
        self.reader = LighthouseSweepAngleReader(self.cf, self._ready_single_sample_cb)
        self.bs_seen = set()

    def extra_layout_field(self):
        h_box = QtWidgets.QHBoxLayout()
        self.seconds_explanation_text = QtWidgets.QLabel()
        self.fill_record_times_line_edit = QtWidgets.QLineEdit(str(DEFAULT_RECORD_TIME))
        self.seconds_explanation_text.setText('Enter the number of seconds you want to record:')
        h_box.addStretch()
        h_box.addWidget(self.seconds_explanation_text)
        h_box.addWidget(self.fill_record_times_line_edit)
        h_box.addStretch()
        self.layout.addLayout(h_box)

    def _record_timer_cb(self):
        self.record_time_current += 1
        self.status_text.setText(self.str_pad('Collecting sweep angles...' +
                                 f' seconds remaining: {self.record_time_total-self.record_time_current}'))

        if self.record_time_current == self.record_time_total:
            self.reader.stop()
            self.status_text.setText(self.str_pad(
                'Recording Done!'+f' Got {len(self.recorded_angles_result)} samples!'))
            self.start_action_button.setText("Restart measurements")
            self.start_action_button.setDisabled(False)
            self.is_done = True
            self.completeChanged.emit()
            self.record_timer.stop()

    def _action_btn_clicked(self):
        self.is_done = False
        self.reader.start()
        self.record_time_current = 0
        self.record_time_total = int(self.fill_record_times_line_edit.text())
        self.record_timer.start(1000)
        self.status_text.setText(self.str_pad('Collecting sweep angles...' +
                                 f' seconds remaining: {self.record_time_total}'))

        self.start_action_button.setDisabled(True)

    def _ready_single_sample_cb(self, bs_id: int, angles: LighthouseBsVectors):
        now = time.time()
        measurement = LhMeasurement(timestamp=now, base_station_id=bs_id, angles=angles)
        self.recorded_angles_result.append(measurement)
        self.bs_seen.add(str(bs_id + 1))

    def get_samples(self):
        return self.recorded_angles_result


class EstimateGeometryThread(QtCore.QObject):
    finished = QtCore.pyqtSignal()
    failed = QtCore.pyqtSignal()

    def __init__(self, origin, x_axis, xy_plane, samples):
        super(EstimateGeometryThread, self).__init__()

        self.origin = origin
        self.x_axis = x_axis
        self.xy_plane = xy_plane
        self.samples = samples
        self.bs_poses = {}

    def run(self):
        try:
            self.bs_poses = self._estimate_geometry(self.origin, self.x_axis, self.xy_plane, self.samples)
            self.finished.emit()
        except Exception as ex:
            print(ex)
            self.failed.emit()

    def get_poses(self):
        return self.bs_poses

    def _estimate_geometry(self, origin: LhCfPoseSample,
                           x_axis: list[LhCfPoseSample],
                           xy_plane: list[LhCfPoseSample],
                           samples: list[LhCfPoseSample]) -> dict[int, Pose]:
        """Estimate the geometry of the system based on samples recorded by a Crazyflie"""
        matched_samples = [origin] + x_axis + xy_plane + LighthouseSampleMatcher.match(samples, min_nr_of_bs_in_match=2)
        initial_guess, cleaned_matched_samples = LighthouseInitialEstimator.estimate(matched_samples,
                                                                                     LhDeck4SensorPositions.positions)

        solution = LighthouseGeometrySolver.solve(initial_guess,
                                                  cleaned_matched_samples,
                                                  LhDeck4SensorPositions.positions)
        if not solution.success:
            raise Exception("No lighthouse base station geometry solution could be found!")

        start_x_axis = 1
        start_xy_plane = 1 + len(x_axis)
        origin_pos = solution.cf_poses[0].translation
        x_axis_poses = solution.cf_poses[start_x_axis:start_x_axis + len(x_axis)]
        x_axis_pos = list(map(lambda x: x.translation, x_axis_poses))
        xy_plane_poses = solution.cf_poses[start_xy_plane:start_xy_plane + len(xy_plane)]
        xy_plane_pos = list(map(lambda x: x.translation, xy_plane_poses))

        # Align the solution
        bs_aligned_poses, transformation = LighthouseSystemAligner.align(
            origin_pos, x_axis_pos, xy_plane_pos, solution.bs_poses)

        cf_aligned_poses = list(map(transformation.rotate_translate_pose, solution.cf_poses))

        # Scale the solution
        bs_scaled_poses, cf_scaled_poses, scale = LighthouseSystemScaler.scale_fixed_point(bs_aligned_poses,
                                                                                           cf_aligned_poses,
                                                                                           [REFERENCE_DIST, 0, 0],
                                                                                           cf_aligned_poses[1])

        return bs_scaled_poses


class EstimateBSGeometryPage(LighthouseBasestationGeometryWizardBasePage):
    def __init__(self, cf: Crazyflie, origin_page: RecordOriginSamplePage, xaxis_page: RecordXAxisSamplePage,
                 xyplane_page: RecordXYPlaneSamplesPage, xyzspace_page: RecordXYZSpaceSamplesPage, parent=None):

        super(EstimateBSGeometryPage, self).__init__(cf)
        self.explanation_text.setText('Step 5. Press the button to estimate the geometry and check the result.\n' +
                                      'If the positions of the base stations look reasonable, press finish to close ' +
                                      'the wizard,\n' +
                                      'if not restart the wizard.')
        pixmap = QtGui.QPixmap(cfclient.module_path + "/ui/wizards/bslh_5.png")
        pixmap = pixmap.scaledToWidth(640)
        self.explanation_picture.setPixmap(pixmap)
        self.start_action_button.setText('Estimate Geometry')
        self.origin_page = origin_page
        self.xaxis_page = xaxis_page
        self.xyplane_page = xyplane_page
        self.xyzspace_page = xyzspace_page
        self.bs_poses = {}

    def _action_btn_clicked(self):
        self.start_action_button.setDisabled(True)
        self.status_text.setText(self.str_pad('Estimating geometry...'))
        origin = self.origin_page.get_sample()
        x_axis = [self.xaxis_page.get_sample()]
        xy_plane = self.xyplane_page.get_samples()
        samples = self.xyzspace_page.get_samples()
        self.thread_estimator = QtCore.QThread()
        self.worker = EstimateGeometryThread(origin, x_axis, xy_plane, samples)
        self.worker.moveToThread(self.thread_estimator)
        self.thread_estimator.started.connect(self.worker.run)
        self.worker.finished.connect(self.thread_estimator.quit)
        self.worker.finished.connect(self._geometry_estimated_finished)
        self.worker.failed.connect(self._geometry_estimated_failed)
        self.worker.finished.connect(self.worker.deleteLater)
        self.thread_estimator.finished.connect(self.thread_estimator.deleteLater)
        self.thread_estimator.start()

    def _geometry_estimated_finished(self):
        self.bs_poses = self.worker.get_poses()
        self.start_action_button.setDisabled(False)
        self.status_text.setText(self.str_pad('Geometry estimated! (X,Y,Z) in meters \n' +
                                 self._print_base_stations_poses(self.bs_poses)))
        self.is_done = True
        self.completeChanged.emit()

    def _geometry_estimated_failed(self):
        self.bs_poses = self.worker.get_poses()
        self.status_text.setText(self.str_pad('Geometry estimate failed! \n' +
                                              'Hit Cancel to close the wizard and start again'))

    def _print_base_stations_poses(self, base_stations: dict[int, Pose]):
        """Pretty print of base stations pose"""
        bs_string = ''
        for bs_id, pose in sorted(base_stations.items()):
            pos = pose.translation
            temp_string = f'    {bs_id + 1}: ({pos[0]}, {pos[1]}, {pos[2]})'
            bs_string += '\n' + temp_string

        return bs_string

    def get_geometry(self):
        geo_dict = {}
        for bs_id, pose in self.bs_poses.items():
            geo = LighthouseBsGeometry()
            geo.origin = pose.translation.tolist()
            geo.rotation_matrix = pose.rot_matrix.tolist()
            geo.valid = True
            geo_dict[bs_id] = geo

        return geo_dict


if __name__ == '__main__':
    import sys
    app = QtWidgets.QApplication(sys.argv)
    wizard = LighthouseBasestationGeometryWizard()
    wizard.show()
    sys.exit(app.exec())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#     ||          ____  _ __
#  +------+      / __ )(_) /_______________ _____  ___
#  | 0xBC |     / __  / / __/ ___/ ___/ __ `/_  / / _ \
#  +------+    / /_/ / / /_/ /__/ /  / /_/ / / /_/  __/
#   ||  ||    /_____/_/\__/\___/_/   \__,_/ /___/\___/
#
#  Copyright (C) 2011-2013 Bitcraze AB
#
#  Crazyflie Nano Quadcopter Client
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA  02110-1301, USA.

"""
Interface for reading input devices and interfaces
"""

from time import time
import logging

logger = logging.getLogger(__name__)


class _ToggleState(dict):

    def __getattr__(self, attr):
        return self.get(attr)

    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__


class InputData:

    def __init__(self):
        # self._toggled = {}
        self._axes = ("roll", "pitch", "yaw", "thrust")
        self._buttons = ("pitchNeg", "pitchPos", "rollNeg", "rollPos",
                         "assistedControl", "estop", "arm",
                         "exitapp", "alt1", "alt2", "muxswitch")
        for axis in self._axes:
            self.__dict__[axis] = 0.0
        self.toggled = _ToggleState()
        self._prev_btn_values = {}
        for button in self._buttons:
            self.__dict__[button] = False
            self.toggled[button] = False
            self._prev_btn_values[button] = False

    def get_all_indicators(self):
        return self._axes + self._buttons

    def _check_toggle(self, key, data):
        if key not in self._prev_btn_values:
            self._prev_btn_values[key] = data
        elif self._prev_btn_values[key] != data:
            self._prev_btn_values[key] = data
            return True
        return False

    def reset_axes(self):
        for axis in self._axes:
            self.__dict__[axis] = 0.0

    def reset_buttons(self):
        for button in self._buttons:
            self.__dict__[button] = False

    def set(self, name, value):
        try:
            if name in self._buttons:
                if self._check_toggle(name, value):
                    self.toggled[name] = True
                else:
                    self.toggled[name] = False
        except KeyError:
            pass
        self.__dict__[name] = value

    def get(self, name):
        return self.__dict__[name]


class InputReaderInterface(object):

    def __init__(self, dev_name, dev_id, dev_reader):
        """Initialize the reader"""
        # Set if the device supports mapping and can be configured
        self.supports_mapping = True

        # Set if the MUX should automatically limit roll/pitch/thrust/yaw
        # according to the settings in the UI
        self.limit_rp = True
        self.limit_thrust = True
        self.limit_yaw = True

        self.input = None

        self._reader = dev_reader
        self.id = dev_id
        self.name = dev_name
        self.input_map = None
        self.input_map_name = ""
        self.data = None
        self._prev_pressed = None
        self.reader_name = dev_reader.name

        self.data = InputData()

        # Stateful things
        self._old_thrust = 0
        self._old_raw_thrust = 0

        self._prev_thrust = 0
        self._last_time = 0
        # How low you have to pull the thrust to bypass the slew-rate (0-100%)
        self.thrust_stop_limit = -90

    def open(self):
        """Initialize the reading and open the device with deviceId and set the
        mapping for axis/buttons using the inputMap"""
        return

    def read(self):
        """Read input from the selected device."""
        return None

    def close(self):
        return

    @staticmethod
    def devices():
        """List all the available devices."""
        return []

    def _cap_rp(self, rp):
        ret = rp * self.input.max_rp_angle
        if ret > self.input.max_rp_angle:
            ret = self.input.max_rp_angle
        elif ret < -1 * self.input.max_rp_angle:
            ret = -1 * self.input.max_rp_angle

        return ret

    def _scale_rp(self, roll, pitch):
        return [self._cap_rp(roll), self._cap_rp(pitch)]

    def _scale_and_deadband_yaw(self, yaw):
        return (InputReaderInterface.deadband(yaw, 0.2) *
                self.input.max_yaw_rate)

    def _limit_thrust(self, thrust, assisted_control, emergency_stop):
        # Thrust limiting (slew, minimum and emergency stop)

        current_time = time()
        if self.input.springy_throttle:
            if assisted_control and \
                    (self.input.get_assisted_control() ==
                     self.input.ASSISTED_CONTROL_ALTHOLD or
                     self.input.get_assisted_control() ==
                     self.input.ASSISTED_CONTROL_HEIGHTHOLD or
                     self.input.get_assisted_control() ==
                     self.input.ASSISTED_CONTROL_HOVER):
                thrust = int(round(InputReaderInterface.deadband(thrust, 0.2) *
                                   32767 + 32767))  # Convert to uint16

                # do not drop thrust to 0 after switching hover mode off
                # set previous values for slew limit logic
                self._prev_thrust = self.input.thrust_slew_limit
                self._last_time = current_time

            else:
                # Scale the thrust to percent (it's between 0 and 1)
                thrust *= 100

                # The default action is to just use the thrust...
                limited_thrust = thrust
                if limited_thrust > self.input.max_thrust:
                    limited_thrust = self.input.max_thrust

                # ... but if we are lowering the thrust, check the limit
                if self._prev_thrust > thrust >= self.thrust_stop_limit and \
                        not emergency_stop:
                    # If we are above the limit, then don't use the slew...
                    if thrust > self.input.thrust_slew_limit:
                        limited_thrust = thrust
                    else:
                        # ... but if we are below first check if we "entered"
                        # the limit, then set it to the limit
                        if self._prev_thrust > self.input.thrust_slew_limit:
                            limited_thrust = self.input.thrust_slew_limit
                        else:
                            # If we are "inside" the limit, then lower
                            # according to the rate we have set each iteration
                            lowering = ((current_time - self._last_time) *
                                        self.input.thrust_slew_rate)
                            limited_thrust = self._prev_thrust - lowering
                elif emergency_stop or thrust < self.thrust_stop_limit:
                    # If the thrust have been pulled down or the
                    # emergency stop has been activated then bypass
                    # the slew and force 0
                    self._prev_thrust = 0
                    limited_thrust = 0

                # For the next iteration set the previous thrust to the limited
                # one (will be the slewed thrust if we are slewing)
                self._prev_thrust = limited_thrust

                # Lastly make sure we're following the "minimum" thrust setting
                if limited_thrust < self.input.min_thrust:
                    self._prev_thrust = 0
                    limited_thrust = 0

                self._last_time = current_time

                thrust = limited_thrust
        else:
            thrust = thrust / 2 + 0.5
            if assisted_control and self.input.get_assisted_control() == \
                    self.input.ASSISTED_CONTROL_ALTHOLD:
                thrust = 32767
            else:
                if thrust < -0.90 or emergency_stop:
                    thrust = 0
                else:
                    thrust = self.input.min_thrust + thrust * (
                        self.input.max_thrust -
                        self.input.min_thrust)
                if (self.input.thrust_slew_enabled and
                    self.input.thrust_slew_limit > thrust and not
                        emergency_stop):
                    if self._old_thrust > self.input.thrust_slew_limit:
                        self._old_thrust = self.input.thrust_slew_limit
                    if thrust < (self._old_thrust -
                                 self.input.thrust_slew_rate / 100):
                        thrust = (self._old_thrust -
                                  self.input.thrust_slew_rate / 100)
                    if thrust < -1 or thrust < self.input.min_thrust:
                        thrust = 0

        self._old_thrust = thrust
        self._old_raw_thrust = thrust
        return thrust

    @staticmethod
    def deadband(value, threshold):
        if abs(value) < threshold:
            value = 0
        elif value > 0:
            value -= threshold
        elif value < 0:
            value += threshold
        return value / (1 - threshold)



# -*- coding: utf-8 -*-
#
#     ||          ____  _ __
#  +------+      / __ )(_) /_______________ _____  ___
#  | 0xBC |     / __  / / __/ ___/ ___/ __ `/_  / / _ \
#  +------+    / /_/ / / /_/ /__/ /  / /_/ / / /_/  __/
#   ||  ||    /_____/_/\__/\___/_/   \__,_/ /___/\___/
#
#  Copyright (C) 2015 Bitcraze AB
#
#  Crazyflie Nano Quadcopter Client
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA  02110-1301, USA.

"""
ZMQ server test application. Use CTRL-C to end the application.

NOTE! If connected to a Crazyflie this will power on the motors!
"""

from threading import Thread
import signal
import time
import sys

try:
    import zmq
except ImportError as e:
    raise Exception("ZMQ library probably not installed ({})".format(e))


class _LogThread(Thread):

    def __init__(self, socket, *args):
        super(_LogThread, self).__init__(*args)
        self._socket = socket

    def run(self):
        while True:
            log = self._socket.recv_json()
            if log["event"] == "data":
                print(log)
            if log["event"] == "created":
                print("Created block {}".format(log["name"]))
            if log["event"] == "started":
                print("Started block {}".format(log["name"]))
            if log["event"] == "stopped":
                print("Stopped block {}".format(log["name"]))
            if log["event"] == "deleted":
                print("Deleted block {}".format(log["name"]))


class _ParamThread(Thread):

    def __init__(self, socket, *args):
        super(_ParamThread, self).__init__(*args)
        self._socket = socket

    def run(self):
        while True:
            param = self._socket.recv_json()
            print(param)


class _ConnThread(Thread):

    def __init__(self, socket, *args):
        super(_ConnThread, self).__init__(*args)
        self._socket = socket

    def run(self):
        while True:
            msg = self._socket.recv_json()
            print(msg)


class _CtrlThread(Thread):

    def __init__(self, socket, *args):
        super(_CtrlThread, self).__init__(*args)
        self._socket = socket
        self._thrust_max = 30000
        self._thrust_min = 20000
        self._thrust = self._thrust_min
        self._thrust_step = 100
        self._cmd = {
            "version": 1,
            "roll": 0.0,
            "pitch": 0.0,
            "yaw": 0.0,
            "thrust": 0.0
        }

    def run(self):
        print("Starting to send control commands!")
        while True:
            time.sleep(0.01)
            self._thrust += self._thrust_step
            if (self._thrust >= self._thrust_max or
                    self._thrust <= self._thrust_min):
                self._thrust_step *= -1
            self._cmd["thrust"] = self._thrust
            self._socket.send_json(self._cmd)


signal.signal(signal.SIGINT, signal.SIG_DFL)

SRV_ADDR = "tcp://127.0.0.1"
CF_URI = "radio://0/10/250K"

context = zmq.Context()
client_conn = context.socket(zmq.REQ)
client_conn.connect("{}:2000".format(SRV_ADDR))

log_conn = context.socket(zmq.SUB)
log_conn.connect("{}:2001".format(SRV_ADDR))
log_conn.setsockopt_string(zmq.SUBSCRIBE, u"")

param_conn = context.socket(zmq.SUB)
param_conn.connect("{}:2002".format(SRV_ADDR))
param_conn.setsockopt_string(zmq.SUBSCRIBE, u"")

conn_conn = context.socket(zmq.SUB)
conn_conn.connect("{}:2003".format(SRV_ADDR))
conn_conn.setsockopt_string(zmq.SUBSCRIBE, u"")

ctrl_conn = context.socket(zmq.PUSH)
ctrl_conn.connect("{}:2004".format(SRV_ADDR))

# Start async threads
log_thread = _LogThread(log_conn)
log_thread.start()

param_thread = _ParamThread(param_conn)
param_thread.start()

conn_thread = _ConnThread(conn_conn)
conn_thread.start()

print("Trying unknown command ...", end=' ')
scan_cmd = {
    "version": 1,
    "cmd": "blah"
}
client_conn.send_json(scan_cmd)
resp = client_conn.recv_json()
if resp["status"] != 0:
    print("fail! {}".format(resp["msg"]))
else:
    print("done!")

print("Scanning for Crazyflies ...", end=' ')
scan_cmd = {
    "version": 1,
    "cmd": "scan"
}
client_conn.send_json(scan_cmd)
resp = client_conn.recv_json()
print("done!")
for i in resp["interfaces"]:
    print("\t{} - {}".format(i["uri"], i["info"]))

connect_cmd = {
    "version": 1,
    "cmd": "connect",
    "uri": "{}".format(CF_URI)
}
print("Connecting to {} ...".format(connect_cmd["uri"]), end=' ')
client_conn.send_json(connect_cmd)
resp = client_conn.recv_json()
if resp["status"] != 0:
    print("fail! {}".format(resp["msg"]))
    sys.exit(1)
print("done!")

# Do logging
print("Loggable variables")
for group in resp["log"]:
    print("\t{}".format(group))
    for name in resp["log"][group]:
        print("\t  {} ({})".format(name,
                                   resp["log"][group][name]["type"]))

print("Parameter variables")
for group in resp["param"]:
    print("\t{}".format(group))
    for name in resp["param"][group]:
        print("\t  {} ({}, {})= {}".format(
            name, resp["param"][group][name]["type"],
            resp["param"][group][name]["access"],
            resp["param"][group][name]["value"]))

log_cmd = {
    "version": 1,
    "cmd": "log",
    "action": "create",
    "name": "Test log block",
    "period": 1000,
    "variables": [
        "pm.vbat",
        "stabilizer.roll"
    ]
}
print("Creating logging {} ...".format(log_cmd["name"]), end=' ')
client_conn.send_json(log_cmd)
resp = client_conn.recv_json()
if resp["status"] == 0:
    print("done!")
else:
    print("fail! {}".format(resp["msg"]))

log_cmd = {
    "version": 1,
    "cmd": "log",
    "action": "start",
    "name": "Test log block"
}
print("Starting logging {} ...".format(log_cmd["name"]), end=' ')
client_conn.send_json(log_cmd)
resp = client_conn.recv_json()
if resp["status"] == 0:
    print("done!")
else:
    print("fail!")

param_cmd = {
    "version": 1,
    "cmd": "param",
    "name": "system.selftestPassed",
    "value": True
}

print("Setting param {} to {}...".format(param_cmd["name"],
                                         param_cmd["value"]), end=' ')
client_conn.send_json(param_cmd)
resp = client_conn.recv_json()
if resp["status"] == 0:
    print("done!")
else:
    print("fail! {}".format(resp["msg"]))


# Start sending control commands
ctrl = _CtrlThread(ctrl_conn)
ctrl.start()

# Wait a bit, then stop the logging
time.sleep(5)

log_cmd = {
    "version": 1,
    "cmd": "log",
    "action": "stop",
    "name": "No name",
}
print("Stopping logging {} ...".format(log_cmd["name"]), end=' ')
client_conn.send_json(log_cmd)
resp = client_conn.recv_json()
if resp["status"] == 0:
    print("done!")
else:
    print("fail! {}".format(resp["msg"]))

log_cmd = {
    "version": 1,
    "cmd": "log",
    "action": "stop",
    "name": "Test log block",
}
print("Stopping logging {} ...".format(log_cmd["name"]), end=' ')
client_conn.send_json(log_cmd)
resp = client_conn.recv_json()
if resp["status"] == 0:
    print("done!")
else:
    print("fail!")

log_cmd = {
    "version": 1,
    "cmd": "log",
    "action": "delete",
    "name": "Test log block",
}
print("Deleting logging {} ...".format(log_cmd["name"]), end=' ')
client_conn.send_json(log_cmd)
resp = client_conn.recv_json()
if resp["status"] == 0:
    print("done!")
else:
    print("fail!")


# Wait a bit, then disconnect
time.sleep(5)

connect_cmd = {
    "version": 1,
    "cmd": "disconnect",
    "uri": "{}".format(CF_URI)
}
print("Disconnecting from {} ...".format(connect_cmd["uri"]), end=' ')
client_conn.send_json(connect_cmd)
resp = client_conn.recv_json()
if resp["status"] != 0:
    print("fail! {}".format(resp["msg"]))
    sys.exit(1)
print("done!")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#     ||          ____  _ __
#  +------+      / __ )(_) /_______________ _____  ___
#  | 0xBC |     / __  / / __/ ___/ ___/ __ `/_  / / _ \
#  +------+    / /_/ / / /_/ /__/ /  / /_/ / / /_/  __/
#   ||  ||    /_____/_/\__/\___/_/   \__,_/ /___/\___/
#
#  Copyright (C) 2014 Bitcraze AB
#
#  Crazyflie Nano Quadcopter Client
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
#  02110-1301, USA.
"""
The mux is used to open one or more devices and mix the inputs from all
of them into one "input" for the Crazyflie and UI.
"""
import logging

__author__ = 'Bitcraze AB'
__all__ = ['InputMux']

logger = logging.getLogger(__name__)


class InputMux(object):

    def __init__(self, input_layer):
        self._devs = {"Device": None}
        self.name = "N/A"
        self.input = input_layer

    def _open_new_device(self, dev, role):
        # Silently close device if open as other role
        for r in self._devs:
            if self._devs[r]:
                if self._devs[r] == dev:
                    self._devs[r] = None
                    dev.close()

        # First set role to None to stop reading
        old_dev = self._devs[role]
        self._devs[role] = None
        if old_dev:
            old_dev.close()

        # Open the new device before attaching it to a role
        dev.open()
        self._devs[role] = dev

    def supported_roles(self):
        return list(self._devs.keys())

    def add_device(self, dev, role):
        logger.info("Adding device {} to MUX {}".format(dev.name, self.name))
        self._open_new_device(dev, role)

    def pause(self):
        for d in [key for key in list(self._devs.keys()) if self._devs[key]]:
            self._devs[d].close()

    def devices(self):
        devs = ()
        for d in self._devs:
            if self._devs[d]:
                devs += (self._devs[d], )
        return devs

    def resume(self):
        for d in [key for key in list(self._devs.keys()) if self._devs[key]]:
            self._devs[d].open()

    def close(self):
        """Close down the MUX and close all it's devices"""
        for d in [key for key in list(self._devs.keys()) if self._devs[key]]:
            self._devs[d].close()
            self._devs[d] = None

    def read(self):
        return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#     ||          ____  _ __
#  +------+      / __ )(_) /_______________ _____  ___
#  | 0xBC |     / __  / / __/ ___/ ___/ __ `/_  / / _ \
#  +------+    / /_/ / / /_/ /__/ /  / /_/ / / /_/  __/
#   ||  ||    /_____/_/\__/\___/_/   \__,_/ /___/\___/
#
#  Copyright (C) 2011-2023 Bitcraze AB
#
#  Crazyflie Nano Quadcopter Client
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  You should have received a copy of the GNU General Public License along with
#  this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""
The bootloader dialog is used to update the Crazyflie firmware and to
read/write the configuration block in the Crazyflie flash.
"""
import logging

import cfclient
from cflib.crazyflie.mem import MemoryElement

from PyQt6 import QtWidgets
from PyQt6 import uic
from PyQt6.QtCore import pyqtSignal

__author__ = 'Bitcraze AB'
__all__ = ['Cf2ConfigDialog']

logger = logging.getLogger(__name__)

service_dialog_class = uic.loadUiType(cfclient.module_path +
                                      "/ui/dialogs/cf2config.ui")[0]


class Cf2ConfigDialog(QtWidgets.QWidget, service_dialog_class):
    """Tab for update the Crazyflie firmware and for reading/writing the config
    block in flash"""

    connected_signal = pyqtSignal(str)
    disconnected_signal = pyqtSignal(str)

    def __init__(self, helper, *args):
        super(Cf2ConfigDialog, self).__init__(*args)
        self.setupUi(self)

        self._cf = helper.cf

        self.disconnected_signal.connect(self._set_ui_disconnected)
        self.connected_signal.connect(self._set_ui_connected)
        self._cf.disconnected.add_callback(self.disconnected_signal.emit)
        self._cf.connected.add_callback(self.connected_signal.emit)

        self._exit_btn.clicked.connect(self.hide)
        self._write_data_btn.clicked.connect(self._write_data)

    def _write_done(self, mem, addr):
        self._cf.mem.get_mems(
            MemoryElement.TYPE_I2C)[0].update(self._data_updated)

    def _data_updated(self, mem):
        self._roll_trim.setValue(mem.elements["roll_trim"])
        self._pitch_trim.setValue(mem.elements["pitch_trim"])
        self._radio_channel.setValue(mem.elements["radio_channel"])
        self._radio_speed.setCurrentIndex(mem.elements["radio_speed"])
        if "radio_address" in mem.elements:
            self._radio_address.setValue(mem.elements["radio_address"])
        else:
            self._radio_address.setValue(int("0xE7E7E7E7E7", 0))
        self._radio_address.setEnabled(True)
        self._write_data_btn.setEnabled(True)

    def _set_ui_connected(self, link_uri):
        mems = self._cf.mem.get_mems(MemoryElement.TYPE_I2C)
        if len(mems) > 0:
            mems[0].update(self._data_updated)

    def _set_ui_disconnected(self, link_uri):
        self._write_data_btn.setEnabled(False)
        self._roll_trim.setValue(0)
        self._pitch_trim.setValue(0)
        self._radio_channel.setValue(0)
        self._radio_speed.setCurrentIndex(0)
        self._radio_address.setValue(0)
        self._radio_address.setEnabled(False)

    def _write_data(self):
        self._write_data_btn.setEnabled(False)
        mem = self._cf.mem.get_mems(MemoryElement.TYPE_I2C)[0]
        mem.elements["pitch_trim"] = self._pitch_trim.value()
        mem.elements["roll_trim"] = self._roll_trim.value()
        mem.elements["radio_channel"] = self._radio_channel.value()
        mem.elements["radio_speed"] = self._radio_speed.currentIndex()
        if "radio_address" in mem.elements:
            mem.elements["radio_address"] = self._radio_address.value()
        mem.write_data(self._write_done)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#     ||          ____  _ __
#  +------+      / __ )(_) /_______________ _____  ___
#  | 0xBC |     / __  / / __/ ___/ ___/ __ `/_  / / _ \
#  +------+    / /_/ / / /_/ /__/ /  / /_/ / / /_/  __/
#   ||  ||    /_____/_/\__/\___/_/   \__,_/ /___/\___/
#
#  Copyright (C) 2014 Bitcraze AB
#
#  Crazyflie Nano Quadcopter Client
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA  02110-1301, USA.

"""
Find all the available input interfaces and try to initialize them.

"""

import logging
from ..inputreaderinterface import InputReaderInterface

__author__ = 'Bitcraze AB'
__all__ = ['InputInterface']

logger = logging.getLogger(__name__)

# Force py2exe to include interfaces module in the build
try:
    from . import leapmotion  # noqa
    from . import wiimote  # noqa
    from . import zmqpull  # noqa
except Exception:
    pass

# Statically listing input interfaces
input_interface = ["leapmotion",
                   "wiimote",
                   "zmqpull"]

logger.info("Found interfaces: {}".format(input_interface))

initialized_interfaces = []
available_interfaces = []

for interface in input_interface:
    try:
        module = __import__(interface, globals(), locals(), [interface], 1)
        main_name = getattr(module, "MODULE_MAIN")
        initialized_interfaces.append(getattr(module, main_name)())
        logger.info("Successfully initialized [{}]".format(interface))
    except Exception as e:
        logger.info("Could not initialize [{}]: {}".format(interface, e))


def devices():
    # Todo: Support rescanning and adding/removing devices
    if len(available_interfaces) == 0:
        for reader in initialized_interfaces:
            devs = reader.devices()
            for dev in devs:
                available_interfaces.append(InputInterface(
                    dev["name"], dev["id"], reader))
    return available_interfaces


class InputInterface(InputReaderInterface):

    def __init__(self, dev_name, dev_id, dev_reader):
        super(InputInterface, self).__init__(dev_name, dev_id, dev_reader)

        # These devices cannot be mapped and configured
        self.supports_mapping = False

        # Ask the reader if it wants to limit
        # roll/pitch/yaw/thrust for all devices
        self.limit_rp = dev_reader.limit_rp
        self.limit_thrust = dev_reader.limit_thrust
        self.limit_yaw = dev_reader.limit_yaw

    def open(self):
        self._reader.open(self.id)

    def close(self):
        self._reader.close(self.id)

    def read(self, include_raw=False):
        mydata = self._reader.read(self.id)
        # Merge interface returned data into InputReader Data Item
        for key in list(mydata.keys()):
            self.data.set(key, mydata[key])

        return self.data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#     ||          ____  _ __
#  +------+      / __ )(_) /_______________ _____  ___
#  | 0xBC |     / __  / / __/ ___/ ___/ __ `/_  / / _ \
#  +------+    / /_/ / / /_/ /__/ /  / /_/ / / /_/  __/
#   ||  ||    /_____/_/\__/\___/_/   \__,_/ /___/\___/
#
#  Copyright (C) 2014 Bitcraze AB
#
#  Crazyflie Nano Quadcopter Client
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
#  USA.

"""
Input interface that supports receiving commands via ZMQ.
"""

import logging
from threading import Thread

from cfclient.utils.config import Config

try:
    import zmq
except Exception as e:
    raise Exception("ZMQ library probably not installed ({})".format(e))

if not Config().get("enable_zmq_input"):
    raise Exception("ZMQ input disabled in config file")

__author__ = 'Bitcraze AB'
__all__ = ['ZMQReader']

ZMQ_PULL_PORT = 1024 + 188

logger = logging.getLogger(__name__)

MODULE_MAIN = "ZMQReader"
MODULE_NAME = "ZMQ"


class _PullReader(Thread):

    def __init__(self, receiver, callback, *args):
        super(_PullReader, self).__init__(*args)
        self._receiver = receiver
        self._cb = callback
        self.daemon = True

    def run(self):
        while True:
            self._cb(self._receiver.recv_json())


class ZMQReader:
    """Used for reading data from input devices using the PyGame API."""

    def __init__(self):
        context = zmq.Context()
        receiver = context.socket(zmq.PULL)
        self._bind_addr = "tcp://127.0.0.1:{}".format(ZMQ_PULL_PORT)
        # If the port is already bound an exception will be thrown
        # and caught in the initialization of the readers and handled.
        receiver.bind(self._bind_addr)
        logger.info("Biding ZMQ at {}".format(self._bind_addr))

        self.name = MODULE_NAME

        self.limit_rp = False
        self.limit_thrust = False
        self.limit_yaw = False

        self.data = {"roll": 0.0, "pitch": 0.0, "yaw": 0.0,
                     "thrust": -1.0, "estop": False, "exit": False,
                     "assistedControl": False, "alt1": False, "alt2": False,
                     "pitchNeg": False, "rollNeg": False,
                     "pitchPos": False, "rollPos": False}

        logger.info("Initialized ZMQ")

        self._receiver_thread = _PullReader(receiver, self._cmd_callback)
        self._receiver_thread.start()

    def _cmd_callback(self, cmd):
        for k in list(cmd["ctrl"].keys()):
            self.data[k] = cmd["ctrl"][k]

    def open(self, device_id):
        """
        Initialize the reading and open the device with deviceId and set the
        mapping for axis/buttons using the inputMap
        """
        return

    def read(self, device_id):
        """Read input from the selected device."""

        return self.data

    def close(self, device_id):
        return

    def devices(self):
        """List all the available connections"""
        # As a temporary workaround we always say we have ZMQ
        # connected. If it's not connected, there's just no data.
        return [{"id": 0, "name": "ZMQ@{}".format(self._bind_addr)}]



# -*- coding: utf-8 -*-
#
#     ||          ____  _ __
#  +------+      / __ )(_) /_______________ _____  ___
#  | 0xBC |     / __  / / __/ ___/ ___/ __ `/_  / / _ \
#  +------+    / /_/ / / /_/ /__/ /  / /_/ / / /_/  __/
#   ||  ||    /_____/_/\__/\___/_/   \__,_/ /___/\___/
#
#  Copyright (C) 2013 Bitcraze AB
#
#  Crazyflie Nano Quadcopter Client
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA  02110-1301, USA.
"""
Headless client for the Crazyflie.
"""
import logging
import os
import signal
import sys

import cfclient.utils
import cflib.crtp
from cfclient.utils.input import JoystickReader
from cflib.crazyflie import Crazyflie

if os.name == 'posix':
    print('Disabling standard output for libraries!')
    stdout = os.dup(1)
    os.dup2(os.open('/dev/null', os.O_WRONLY), 1)
    sys.stdout = os.fdopen(stdout, 'w')

# set SDL to use the dummy NULL video driver,
#   so it doesn't need a windowing system.
os.environ["SDL_VIDEODRIVER"] = "dummy"


class HeadlessClient():
    """Crazyflie headless client"""

    def __init__(self):
        """Initialize the headless client and libraries"""
        cflib.crtp.init_drivers()

        self._jr = JoystickReader(do_device_discovery=False)

        self._cf = Crazyflie(ro_cache=None,
                             rw_cache=cfclient.config_path + "/cache")

        signal.signal(signal.SIGINT, signal.SIG_DFL)

        self._devs = []

        for d in self._jr.available_devices():
            self._devs.append(d.name)

    def setup_controller(self, input_config, input_device=0):
        """Set up the device reader"""
        # Set up the joystick reader
        devs = self._jr.available_devices()  # noqa, is this a bug?
        print("Will use [%s] for input" % self._devs[input_device])
        self._jr.start_input(self._devs[input_device])
        self._jr.set_input_map(self._devs[input_device], input_config)

    def controller_connected(self):
        """ Return True if a controller is connected"""
        return True if (len(self._jr.available_devices()) > 0) else False

    def list_controllers(self):
        """List the available controllers and input mapping"""
        print("\nAvailable controllers:")
        for i, dev in enumerate(self._devs):
            print(" - Controller #{}: {}".format(i, dev))
        print("\nAvailable input mapping:")
        for map in os.listdir(cfclient.config_path + '/input'):
            print(" - " + map.split(".json")[0])

    def connect_crazyflie(self, link_uri):
        """Connect to a Crazyflie on the given link uri"""
        self._cf.connection_failed.add_callback(self._connection_failed)
        # 2014-11-25 chad: Add a callback for when we have a good connection.
        self._cf.connected.add_callback(self._connected)
        self._cf.param.add_update_callback(
            group="imu_sensors", name="AK8963", cb=(
                lambda name, found: self._jr.set_alt_hold_available(
                    eval(found))))
        self._jr.assisted_control_updated.add_callback(
            lambda enabled: self._cf.param.set_value("flightmode.althold",
                                                     enabled))

        self._cf.open_link(link_uri)
        self._jr.input_updated.add_callback(self._cf.commander.send_setpoint)

    def _connected(self, link):
        """Callback for a successful Crazyflie connection."""
        print("Connected to {}".format(link))

    def _connection_failed(self, link, message):
        """Callback for a failed Crazyflie connection"""
        print("Connection failed on {}: {}".format(link, message))
        sys.exit(-1)

    def _input_dev_error(self, message):
        """Callback for an input device error"""
        print("Error when reading device: {}".format(message))
        sys.exit(-1)


def main():
    """Main Crazyflie headless application"""
    import argparse

    parser = argparse.ArgumentParser(prog="cfheadless")
    parser.add_argument("-u", "--uri", action="store", dest="uri", type=str,
                        default="radio://0/10/250K",
                        help="URI to use for connection to the Crazyradio"
                             " dongle, defaults to radio://0/10/250K")
    parser.add_argument("-i", "--input", action="store", dest="input",
                        type=str, default="PS3_Mode_1",
                        help="Input mapping to use for the controller,"
                             "defaults to PS3_Mode_1")
    parser.add_argument("-d", "--debug", action="store_true", dest="debug",
                        help="Enable debug output")
    parser.add_argument("-c", "--controller", action="store", type=int,
                        dest="controller", default=0,
                        help="Use controller with specified id,"
                             " id defaults to 0")
    parser.add_argument("--controllers", action="store_true",
                        dest="list_controllers",
                        help="Only display available controllers and exit")
    (args, unused) = parser.parse_known_args()

    if args.debug:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)

    headless = HeadlessClient()

    if (args.list_controllers):
        headless.list_controllers()
    else:
        if headless.controller_connected():
            headless.setup_controller(input_config=args.input,
                                      input_device=args.controller)
            headless.connect_crazyflie(link_uri=args.uri)
        else:
            print("No input-device connected, exiting!")


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#     ||          ____  _ __
#  +------+      / __ )(_) /_______________ _____  ___
#  | 0xBC |     / __  / / __/ ___/ ___/ __ `/_  / / _ \
#  +------+    / /_/ / / /_/ /__/ /  / /_/ / / /_/  __/
#   ||  ||    /_____/_/\__/\___/_/   \__,_/ /___/\___/
#
#  Copyright (C) 2011-2017 Bitcraze AB
#
#  Crazyflie Nano Quadcopter Client
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  You should have received a copy of the GNU General Public License along with
#  this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""
Find all the available tabs so they can be loaded.

Dropping a new .py file into this directory will automatically list and load
it into the UI when it is started.
"""
from .ConsoleTab import ConsoleTab
from .CrtpSharkToolbox import CrtpSharkToolbox
# from .ExampleTab import ExampleTab
from .FlightTab import FlightTab
# from .GpsTab import GpsTab
from .LEDRingTab import LEDRingTab
from .LogBlockTab import LogBlockTab
from .LogTab import LogTab
from .ParamTab import ParamTab
from .PlotTab import PlotTab
from .locopositioning_tab import LocoPositioningTab
from .LogClientTab import LogClientTab
from .lighthouse_tab import LighthouseTab
from .TuningTab import TuningTab
from .ColorLEDTab import ColorLEDTab

__author__ = 'Bitcraze AB'
__all__ = []

available = [
    ConsoleTab,
    # ExampleTab,
    FlightTab,
    # GpsTab,
    LEDRingTab,
    ColorLEDTab,
    LogBlockTab,
    LogTab,
    ParamTab,
    PlotTab,
    LocoPositioningTab,
    LighthouseTab,
    LogClientTab,
    TuningTab,
    CrtpSharkToolbox,
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#     ||          ____  _ __
#  +------+      / __ )(_) /_______________ _____  ___
#  | 0xBC |     / __  / / __/ ___/ ___/ __ `/_  / / _ \
#  +------+    / /_/ / / /_/ /__/ /  / /_/ / / /_/  __/
#   ||  ||    /_____/_/\__/\___/_/   \__,_/ /___/\___/
#
#  Copyright (C) 2013-2017 Bitcraze AB
#  Copyright (C) 2013 Allyn Bauer
#
#  Crazyflie Nano Quadcopter Client
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA  02110-1301, USA.

"""
Manager for loading/accesing input device mappings.
"""

import json
import logging
import glob
import os
import copy

from .singleton import Singleton
from cflib.utils.callbacks import Caller

import cfclient

__author__ = 'Bitcraze AB/Allyn Bauer'
__all__ = ['ConfigManager']

logger = logging.getLogger(__name__)


class ConfigManager(metaclass=Singleton):
    """ Singleton class for managing input processing """
    conf_needs_reload = Caller()
    configs_dir = cfclient.config_path + "/input"

    def __init__(self):
        """Initialize and create empty config list"""
        self._list_of_configs = []

    def get_config(self, config_name):
        """Get the button and axis mappings for an input device."""
        try:
            idx = self._list_of_configs.index(config_name)
            return self._input_config[idx]
        except Exception:
            return None

    def get_settings(self, config_name):
        """Get the settings for an input device."""
        try:
            idx = self._list_of_configs.index(config_name)
            return self._input_settings[idx]
        except Exception:
            return None

    def get_list_of_configs(self):
        """Reload the configurations from file"""
        try:
            configs = [os.path.basename(f) for f in
                       glob.glob(self.configs_dir + "/[A-Za-z]*.json")]
            self._input_config = []
            self._input_settings = []
            self._list_of_configs = []
            for conf in configs:
                logger.debug("Parsing [%s]", conf)
                json_data = open(self.configs_dir + "/%s" % conf)
                data = json.load(json_data)
                new_input_device = {}
                new_input_settings = {"updateperiod": 10,
                                      "springythrottle": True,
                                      "rp_dead_band": 0.05}
                for s in data["inputconfig"]["inputdevice"]:
                    if s == "axis":
                        for a in data["inputconfig"]["inputdevice"]["axis"]:
                            axis = {}
                            axis["scale"] = a["scale"]
                            axis["offset"] = a[
                                "offset"] if "offset" in a else 0.0
                            axis["type"] = a["type"]
                            axis["key"] = a["key"]
                            axis["name"] = a["name"]

                            self._translate_for_backwards_compatibility(axis)

                            try:
                                ids = a["ids"]
                            except Exception:
                                ids = [a["id"]]
                            for id in ids:
                                locaxis = copy.deepcopy(axis)
                                if "ids" in a:
                                    if id == a["ids"][0]:
                                        locaxis["scale"] = locaxis[
                                            "scale"] * -1
                                locaxis["id"] = id
                                # 'type'-'id' defines unique index for axis
                                index = "%s-%d" % (a["type"], id)
                                new_input_device[index] = locaxis
                    else:
                        new_input_settings[s] = data[
                            "inputconfig"]["inputdevice"][s]
                self._input_config.append(new_input_device)
                self._input_settings.append(new_input_settings)
                json_data.close()
                self._list_of_configs.append(conf[:-5])
        except Exception as e:
            logger.warning("Exception while parsing inputconfig file: %s ", e)
        return self._list_of_configs

    def save_config(self, input_map, config_name):
        """Save a configuration to file"""
        mapping = {'inputconfig': {'inputdevice': {'axis': []}}}

        # Create intermediate structure for the configuration file
        funcs = {}
        for m in input_map:
            key = input_map[m]["key"]
            if key not in funcs:
                funcs[key] = []
            funcs[key].append(input_map[m])

        # Create a mapping for each axis, take care to handle
        # split axis configurations
        for a in funcs:
            func = funcs[a]
            axis = {}
            # Check for split axis
            if len(func) > 1:
                axis["ids"] = [func[0]["id"], func[1]["id"]]
                axis["scale"] = func[1]["scale"]
            else:
                axis["id"] = func[0]["id"]
                axis["scale"] = func[0]["scale"]
            axis["key"] = func[0]["key"]
            axis["name"] = func[0]["key"]  # Name isn't used...
            axis["type"] = func[0]["type"]
            mapping["inputconfig"]["inputdevice"]["axis"].append(axis)

        mapping["inputconfig"]['inputdevice']['name'] = config_name
        mapping["inputconfig"]['inputdevice']['updateperiod'] = 10

        filename = ConfigManager().configs_dir + "/%s.json" % config_name
        logger.info("Saving config to [%s]", filename)
        json_data = open(filename, 'w')
        json_data.write(json.dumps(mapping, indent=2))
        json_data.close()

        self.conf_needs_reload.call(config_name)

    def _translate_for_backwards_compatibility(self, axis):
        """Handle changes in the config file format"""

        # The parameter that used to be called 'althold' has been renamed to
        # 'assistedControl'
        althold = 'althold'
        assistedControl = 'assistedControl'

        if axis['key'] == althold:
            axis['key'] = assistedControl

        if axis['name'] == althold:
            axis['name'] = assistedControl



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#     ||          ____  _ __
#  +------+      / __ )(_) /_______________ _____  ___
#  | 0xBC |     / __  / / __/ ___/ ___/ __ `/_  / / _ \
#  +------+    / /_/ / / /_/ /__/ /  / /_/ / / /_/  __/
#   ||  ||    /_____/_/\__/\___/_/   \__,_/ /___/\___/
#
#  Copyright (C) 2011-2023 Bitcraze AB
#
#  Crazyflie Nano Quadcopter Client
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  You should have received a copy of the GNU General Public License along with
#  this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""
Toolbox used to interact with the Basestation to set it in a certain channel
"""

from PyQt6 import QtWidgets
from PyQt6 import uic
from PyQt6.QtCore import Qt

import io
import serial
from serial.tools.list_ports import comports
import cfclient
import time


__author__ = 'Bitcraze AB'
__all__ = ['LighthouseBsModeDialog']

(basestation_mode_widget_class, connect_widget_base_class) = uic.loadUiType(
    cfclient.module_path + "/ui/dialogs/basestation_mode_dialog.ui")


class LighthouseBsModeDialog(QtWidgets.QWidget, basestation_mode_widget_class):

    PID = 0x2500
    VID = 0x28de

    def __init__(self, helper, *args):
        super(LighthouseBsModeDialog, self).__init__(*args)

        self.setupUi(self)

        self.helper = helper

        self._set_basestation_button.pressed.connect(self._set_basestation_pressed)
        self._set_channel_spinbox.valueChanged.connect(self._set_channel_number)
        self._scan_basestation_button.pressed.connect(self._set_basestation_dev)

        self._channel = 1
        self._device = None

        self._basestation_port_display.setText('No basestation found!')

        self._basestation_port_display.setText(self._device)

    def _set_basestation_dev(self):
        self._device = self._find_basestation()
        if self._device is None:
            self._basestation_port_display.setText('No basestation found!')
            self._set_basestation_button.setEnabled(False)
            self._display_current_channel.setText('')
            self._display_current_id.setText('')
        else:
            self._basestation_port_display.setText(self._device)
            self._set_basestation_button.setEnabled(True)
            self._check_current_mode()
            self._check_current_id()

    def _check_current_mode(self):
        dev = self._device
        ser = serial.Serial(dev, timeout=0.4)
        sio = io.TextIOWrapper(io.BufferedRWPair(ser, ser))
        sio.write("\r\nmode \r\n")
        sio.flush()
        mode_confirm_lines = sio.readlines()
        for line in mode_confirm_lines:
            if line.startswith('Current mode: '):
                parts = line.split()
                confirm_mode = int(parts[2])
        if confirm_mode == 0:
            self._display_current_channel.setText('0 (not supported)')
        else:
            self._display_current_channel.setText(str(confirm_mode))
        ser.close()

    def _check_current_id(self):
        dev = self._device
        ser = serial.Serial(dev, timeout=0.4)
        sio = io.TextIOWrapper(io.BufferedRWPair(ser, ser))

        sio.write("\r\nid\r\n")
        sio.flush()
        id_lines = sio.readlines()

        uid = id_lines[3].split(': ')[1].strip()
        self._display_current_id.setText(str(uid))
        ser.close()

    def _set_basestation_pressed(self):
        self._set_basestation_button.setEnabled(False)
        dev = self._device
        try:
            ser = serial.Serial(dev, timeout=0.4)
        except serial.SerialException:
            self._basestation_mode_status.setText(
                'Permission denied: cannot access serial port.\n'
                'Try running: \"sudo usermod -aG dialout [username]\" '
                'and then restart your computer.'
            )

            self._set_basestation_button.setEnabled(True)
            return

        sio = io.TextIOWrapper(io.BufferedRWPair(ser, ser))
        sio.write("\r\nmode " + str(self._channel) + "\r\n")
        sio.flush()
        time.sleep(1)
        sio.write("\r\nparam save\r\n")
        sio.flush()
        time.sleep(1)
        sio.write("\r\nmode\r\n")
        sio.flush()
        mode_confirm_lines = sio.readlines()
        confirm_mode = None
        for line in mode_confirm_lines:
            if line.startswith('Current mode: '):
                parts = line.split()
                confirm_mode = int(parts[2])
        if confirm_mode is self._channel:
            self._basestation_mode_status.setText('Success !')
        else:
            self._basestation_mode_status.setText('Try again !')
        self._display_current_channel.setText(str(confirm_mode))
        self._set_basestation_button.setEnabled(True)
        ser.close()

    def _set_channel_number(self, value):
        self._channel = value

    def getName(self):
        return 'LH Basestation Setup'

    def getTabName(self):
        return 'LH Basestation Setup'

    def enable(self):
        return

    def disable(self):
        return

    def preferedDockArea(self):
        return Qt.DockWidgetArea.RightDockWidgetArea

    def _find_basestation(self):
        ports = comports()

        for port in ports:
            if port.vid == self.VID and port.pid == self.PID:
                return port.device

    def reset(self):
        self._channel = 1
        self._device = None
        self._set_basestation_button.setEnabled(False)
        self._basestation_port_display.setText('No basestation found!')
        self._display_current_channel.setText('')
        self._basestation_mode_status.setText('')
        self._set_channel_spinbox.setValue(self._channel)
        self._display_current_id.setText('')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#     ||          ____  _ __
#  +------+      / __ )(_) /_______________ _____  ___
#  | 0xBC |     / __  / / __/ ___/ ___/ __ `/_  / / _ \
#  +------+    / /_/ / / /_/ /__/ /  / /_/ / / /_/  __/
#   ||  ||    /_____/_/\__/\___/_/   \__,_/ /___/\___/
#
#  Copyright (C) 2011-2023 Bitcraze AB
#
#  Crazyflie Nano Quadcopter Client
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License along with
#  this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""
Superclass for all tabs that implements common functions.
"""

import logging

from PyQt6 import QtWidgets
from PyQt6.QtCore import pyqtSignal
from PyQt6.QtCore import Qt
from PyQt6.QtGui import QCloseEvent

from cfclient.utils.config import Config

__author__ = 'Bitcraze AB'
__all__ = ['TabToolbox']

logger = logging.getLogger(__name__)


class TabToolbox(QtWidgets.QWidget):
    """Superclass for all tabs that implements common functions."""

    CONF_KEY_OPEN_TABS = "open_tabs"
    CONF_KEY_OPEN_TOOLBOXES = "open_toolboxes"
    CONF_KEY_TOOLBOX_AREAS = "toolbox_areas"

    # Display states
    DS_HIDDEN = 0
    DS_TAB = 1
    DS_TOOLBOX = 2

    def __init__(self, helper, tab_toolbox_name):
        super(TabToolbox, self).__init__()
        self._helper = helper
        self.tab_toolbox_name = tab_toolbox_name

        # Dock widget for toolbox behavior
        self.dock_widget = self.ClosingDockWidget(tab_toolbox_name)
        self.dock_widget.tab_toolbox = self

        self._display_state = self.DS_HIDDEN

        self._dock_area = self._get_toolbox_area_config()

        # Do not allow floating toolboxes, it seems to be buggy
        self.dock_widget.setFeatures(QtWidgets.QDockWidget.DockWidgetFeature.DockWidgetClosable |
                                     QtWidgets.QDockWidget.DockWidgetFeature.DockWidgetMovable)
        # If floating is set in the config, change to right docking area
        if self._dock_area == Qt.DockWidgetArea.NoDockWidgetArea:
            self._dock_area = Qt.DockWidgetArea.RightDockWidgetArea

    def get_tab_toolbox_name(self):
        """Return the name that will be shown in the tab or toolbox"""
        return self.tab_toolbox_name

    def is_visible(self):
        return self._display_state != self.DS_HIDDEN

    def get_display_state(self):
        return self._display_state

    def set_display_state(self, new_display_state):
        if new_display_state != self._display_state:
            self._display_state = new_display_state
            self._update_open_config(new_display_state)

            if new_display_state == self.DS_HIDDEN:
                self.disable()
            else:
                self.enable()

    def preferred_dock_area(self):
        return self._dock_area

    def set_preferred_dock_area(self, area):
        self._dock_area = area
        self._store_toolbox_area_config(area)

    def enable(self):
        pass

    def disable(self):
        pass

    @classmethod
    def read_open_tab_config(cls):
        return cls._read_open_config(TabToolbox.CONF_KEY_OPEN_TABS)

    @classmethod
    def read_open_toolbox_config(cls):
        return TabToolbox._read_open_config(TabToolbox.CONF_KEY_OPEN_TOOLBOXES)

    @classmethod
    def _read_open_config(cls, key):
        config = []
        try:
            value = Config().get(key)
            # Python will return a list of an empty string if value is empty, filter it
            config = list(filter(None, value.split(",")))
        except KeyError:
            logger.info(f'No config found for {key}')

        return config

    def _update_open_config(self, display_state):
        if display_state == self.DS_HIDDEN:
            self._remove_from_open_config(TabToolbox.CONF_KEY_OPEN_TABS)
            self._remove_from_open_config(TabToolbox.CONF_KEY_OPEN_TOOLBOXES)
        elif display_state == self.DS_TAB:
            self._add_to_open_config(TabToolbox.CONF_KEY_OPEN_TABS)
            self._remove_from_open_config(TabToolbox.CONF_KEY_OPEN_TOOLBOXES)
        elif display_state == self.DS_TOOLBOX:
            self._remove_from_open_config(TabToolbox.CONF_KEY_OPEN_TABS)
            self._add_to_open_config(TabToolbox.CONF_KEY_OPEN_TOOLBOXES)

    def _add_to_open_config(self, key):
        config = self._read_open_config(key)
        name = self.tab_toolbox_name

        if name not in config:
            config.append(name)
            self._store_open_config(key, config)

    def _remove_from_open_config(self, key):
        config = self._read_open_config(key)
        name = self.tab_toolbox_name

        if name in config:
            config.remove(name)
            self._store_open_config(key, config)

    def _store_open_config(self, key, config):
        value = ','.join(config)
        Config().set(key, value)

    def _get_toolbox_area_config(self):
        result = Qt.DockWidgetArea.RightDockWidgetArea

        config = self._read_toolbox_area_config()

        if self.tab_toolbox_name in config.keys():
            result = Qt.DockWidgetArea(config[self.tab_toolbox_name])

        return result

    def _store_toolbox_area_config(self, area):
        config = self._read_toolbox_area_config()
        config[self.tab_toolbox_name] = area.value
        self._write_toolbox_area_config(config)

    def _read_toolbox_area_config(self):
        composite_config = []
        try:
            key = self.CONF_KEY_TOOLBOX_AREAS
            value = Config().get(key)
            # Python will return a list of an empty string if value is empty, filter it
            composite_config = list(filter(None, value.split(",")))
        except KeyError:
            logger.info(f'No config found for {key}')

        config = {}
        for composite in composite_config:
            try:
                parts = composite.split(':')
                config[parts[0]] = int(parts[1])
            except (KeyError, ValueError):
                logger.info(f'Can not understand config {composite}')

        return config

    def _write_toolbox_area_config(self, config):
        key = self.CONF_KEY_TOOLBOX_AREAS
        value = ','.join(map(lambda item: f'{item[0]}:{item[1]}', config.items()))
        Config().set(key, value)

    class ClosingDockWidget(QtWidgets.QDockWidget):
        closed = pyqtSignal()

        def closeEvent(self, event: QCloseEvent) -> None:
            super(TabToolbox.ClosingDockWidget, self).closeEvent(event)
            self.closed.emit()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Configuration block compiler/decompiler
# Fully static version (to be considered as a prototype...)!
from configparser import ConfigParser
import struct
import sys
from functools import reduce

# Radio speed enum type
speeds = ["250K", "1M", "2M"]
radioSpeedPos = 2

defaultConfig = """#Crazyflie config block
#default version generated from squatch
[radio]
channel= 100
speed= 2M

[calib]
pitchTrim= 0.0
rollTrim= 0.0
"""

config = """#Crazyflie config block
#Block version %d extracted from copter
[radio]
channel= %d
speed= %s

[calib]
pitchTrim= %f
rollTrim= %f
"""

configVersion = 0
structFormat = "<BBBff"


def checksum256(st):
    return reduce(lambda x, y: x + y, list(map(ord, st))) % 256


def compileBlock(configFile, binFile):
    config = ConfigParser()

    config.read(configFile)

    block = (configVersion,)

    block += (config.getint("radio", "channel"),)
    block += (speeds.index(config.get("radio", "speed").upper()),)

    block += (config.getfloat("calib", "pitchTrim"),)
    block += (config.getfloat("calib", "rollTrim"),)

    bin = struct.pack(structFormat, *block)

    # Adding some magic:
    bin = "0xBC" + bin

    bin += struct.pack("B", 256 - checksum256(bin))

    # print("Config block checksum: %02x" % bin[len(bin)-1])

    bfile = open(binFile, "w")
    bfile.write(bin)
    bfile.close()

    print("Config block compiled successfully to", binFile)


def decompileBlock(binFile, configFile):
    bfile = open(binFile)
    bin = bfile.read()
    bfile.close()

    if (bin[0:4] != "0xBC" or len(bin) < (struct.calcsize(structFormat) + 5) or
            checksum256(bin[0:struct.calcsize(structFormat) + 5]) != 0):
        print("Config block erased of altered, generating default file")
        cfile = open(configFile, "w")
        cfile.write(defaultConfig)
        cfile.close()
    else:
        block = struct.unpack(structFormat,
                              bin[4:struct.calcsize(structFormat) + 4])
        if block[0] != configVersion:
            print("Error! wrong configuration block version, "
                  "this program must certainly be updated!")
            return

        block = (block[0:radioSpeedPos] + (speeds[block[radioSpeedPos]],) +
                 block[radioSpeedPos + 1:len(block)])

        cfile = open(configFile, "w")
        cfile.write(config % block)
        cfile.close()
        print("Config block successfully extracted to", configFile)


if __name__ == "__main__":
    if (len(sys.argv) < 4 or
            (sys.argv[1] != "generate" and sys.argv[1] != "extract")):
        print("Configuration block compiler/decompiler.")
        print("  Usage: %s <generate|extract> <infile> <outfile>"
              % sys.argv[0])

    if sys.argv[1] == "generate":
        compileBlock(sys.argv[2], sys.argv[3])
    else:
        decompileBlock(sys.argv[2], sys.argv[3])



# -*- coding: utf-8 -*-
import logging
from threading import Thread
import time

try:
    import cwiid
except ImportError as e:
    raise Exception("Missing cwiid (wiimote) driver {}".format(e))

logger = logging.getLogger(__name__)

MODULE_MAIN = "WiimoteReader"
MODULE_NAME = "WII"


class _Reader(object):
    # needs attributes:
    # - name
    # - limit_rp
    # - limit_thrust
    # - limit_yaw
    # - open

    def devices(self):
        """List all the available connections"""
        raise NotImplementedError

    def open(self, device_id):
        """
        Initialize the reading and open the device with deviceId and set the
        mapping for axis/buttons using the inputMap
        """
        return

    def close(self, device_id):
        return

    def read(self, device_id):
        """Read input from the selected device."""
        raise NotImplementedError


TWO = 1
ONE = 2
B = 4
A = 8
MINUS = 16
HOME = 128
LEFT = 256
RIGHT = 512
DOWN = 1024
UP = 2048
PLUS = 4096


class HandleWiimote(Thread):

    def __init__(self, reader, wii, *args):
        super(HandleWiimote, self).__init__(*args)
        self.reader = reader
        self.wii = wii
        self.daemon = True

    def run(self):
        logger.info("\n\nRUNNING THREAD!\n\n\n")
        t_delta = 100
        move_delta = .3
        max_move = 8
        min_sample = .1
        max_sample = .01
        sample = min_sample
        while True:
            button = self.wii.state['buttons']
            pitch = self.reader.data['pitch']
            roll = self.reader.data['roll']
            if button & ONE:
                logger.info("UP!! {}".format(self.reader.data['thrust']))
                self.reader.data['thrust'] += t_delta
            if button & TWO:
                logger.info("DOWN!! {}".format(self.reader.data['thrust']))
                self.reader.data['thrust'] -= t_delta * 3
                if self.reader.data['thrust'] < -1:
                    self.reader.data['thrust'] = -1
            if button & RIGHT:
                logger.info("RIGHT PITCH {}".format(pitch))
                self.reader.data['pitch'] = min(max_move, pitch + move_delta)
            if button & LEFT:
                logger.info("LEFT PITCH {}".format(pitch))
                self.reader.data['pitch'] = max(-max_move, pitch - move_delta)
            if button & UP:
                logger.info("UP ROLL {}".format(roll))
                self.reader.data['roll'] = max(-max_move, roll - move_delta)
            if button & DOWN:
                logger.info("DOWN ROLL {}".format(roll))
                self.reader.data['roll'] = min(max_move, roll + move_delta)
            if button & B:
                # KILL
                self.reader.data['thrust'] = -1

            if button:
                sample = max(max_sample, sample / 3)
            else:
                sample = min(min_sample, sample * 3)
                self.adjust()
            time.sleep(sample)

    def adjust(self):
        pitch = self.reader.data['pitch']
        if pitch > 1.2:
            self.reader.data['pitch'] -= 1
        elif pitch < -1.2:
            self.reader.data['pitch'] += 1
        else:
            self.reader.data['pitch'] = 0
        roll = self.reader.data['roll']
        if roll > 1.2:
            self.reader.data['roll'] -= 1
        elif roll < -1.2:
            self.reader.data['roll'] += 1
        else:
            self.reader.data['roll'] = 0


class WiimoteReader(_Reader):
    name = MODULE_NAME

    def __init__(self):
        self.limit_rp = False
        self.limit_thrust = False
        self.limit_yaw = False

        print("Press 1 + 2 to connect wii")
        time.sleep(1)
        self.wm = cwiid.Wiimote()
        self.wm.rpt_mode = cwiid.RPT_BTN
        logger.info("FOUND WIIMOTE")
        self.data = {"roll": 0.0, "pitch": 0.0, "yaw": 0.0,
                     "thrust": -1.0, "estop": False, "exit": False,
                     "assistedControl": False, "alt1": False, "alt2": False,
                     "pitchNeg": False, "rollNeg": False,
                     "pitchPos": False, "rollPos": False}
        self.wii_thread = HandleWiimote(self, self.wm)
        self.wii_thread.start()

    def read(self, dev_id):
        return self.data

    def devices(self):
        """List all the available connections"""
        return [{"id": 0, "name": "WII@{}".format(self.wm)}]



# -*- coding: utf-8 -*-
#
#  ,---------,       ____  _ __
#  |  ,-^-,  |      / __ )(_) /_______________ _____  ___
#  | (  O  ) |     / __  / / __/ ___/ ___/ __ `/_  / / _ \
#  | / ,--´  |    / /_/ / / /_/ /__/ /  / /_/ / / /_/  __/
#     +------`   /_____/_/\__/\___/_/   \__,_/ /___/\___/
#
#  Copyright (C) 2020-2023 Bitcraze AB
#
#  Crazyflie Nano Quadcopter Client
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA  02110-1301, USA.

import os
from PyQt6.QtGui import QFont

import cfclient

__author__ = 'Bitcraze AB'
__all__ = ['UiUtils']

ICON_PATH = os.path.join(cfclient.module_path, 'ui', 'icons')


class UiUtils:

    COLOR_GREEN = '#7cdb37'
    COLOR_BLUE = '#3399ff'
    COLOR_RED = '#cc0404'
    COLOR_NAVY_BLUE = '#23335D'
    COLOR_NAVY_LIGHT_BLUE = '#3c4869'
    COLOR_LIGHT_GRAY = '#C8C8C8'
    COLOR_LIGHT_GRAY2 = '#A3A3A3'
    COLOR_HACKER_BLACK = '#0A0B0B'
    COLOR_HACKER_GREEN = '#00FF2F'

    FONT = QFont('Palantino')

    THEMES = ['Default', 'Navy blue', 'Hacker']

    # Note: progress bar styling is required for all themes to make the bars
    # wider on OSX (default is very thin) and display the text in the bar.
    # In general terms, when styling something, do check on all OSes, it
    # rarely looks the same.

    _THEME_DEFAULT = """
            QProgressBar {
                border: 1px solid gray;
                border-radius: 2px;
                text-align: center;
            }

            QProgressBar::chunk {
                border-radius: 2px;
                background-color: """ + COLOR_BLUE + """;
            }
    """

    _THEME_HACKER = """
            QApplication {
                font: Palantino;
            }

            QProgressBar {
                border: 1px solid gray;
                border-radius: 2px;
                background-color: white;
                text-align: center;
            }

            QProgressBar::chunk {
                border-radius: 2px;
                background-color: """ + COLOR_GREEN + """;
            }

            QWidget {
                background-color: """ + COLOR_HACKER_BLACK + """;
                color: white;
            }

            QPushButton {
                background-color: """ + COLOR_HACKER_BLACK + """;
                color: """ + COLOR_HACKER_GREEN + """;
                border: 1px solid gray;
            }

            QPushButton:hover {
                background-color: """ + COLOR_LIGHT_GRAY + """;
            }

            QComboBox, QAbstractSpinBox, QAbstractSpinBox::Dropdown, QSpinBox {
                background-color: white;
                color: """ + COLOR_HACKER_BLACK + """;
            }

            QComboBox:disabled,
            QAbstractSpinBox:disabled,
            QSpinBox:disabled,
            QAbstractButton:disabled {
                color: gray;
            }

            QLineEdit {
                border-style: outset;
                border-width: 2px;
                border-radius: 10px;
                border-color: white;
                background-color: white;
                color: """ + COLOR_HACKER_BLACK + """;
                margin: 3px;
                border-radius: 2px;
            }

            QMenu::item:selected {
                background-color: """ + COLOR_LIGHT_GRAY + """;
            }

            QTabWidget {
                border: 3px solid white;
            }

            QCheckBox::indicator {
                border: 1px solid white;
                background: """ + COLOR_HACKER_BLACK + """;
            }

            QCheckBox::indicator:checked {
                image: url(""" + ICON_PATH + '/checkmark_white.png' + """);
            }

            QComboBox {
                selection-background-color: white;
                selection-color: """ + COLOR_HACKER_GREEN + """;
                color: """ + COLOR_HACKER_GREEN + """;
                background-color: """ + COLOR_HACKER_BLACK + """;
                border: 1px solid white;
            }

            QComboBox, QAbstractItemView {
                color: """ + COLOR_HACKER_GREEN + """;
                background-color: """ + COLOR_HACKER_BLACK + """;
            }

            .QSlider {
                min-width: 100px;
                max-width: 100px;
            }

            .QSlider::groove:vertical {
                border: 1px solid white;
                width: 10px;
            }

            .QSlider::handle:vertical {
                background: """ + COLOR_GREEN + """;
                border: 5px solid #B5E61D;
                height: 5px;
                border-radius: 30px;
            }

            QTreeView, QTextEdit {
                border-style: outset;
                border-width: 1px;
                border-color: """ + COLOR_LIGHT_GRAY2 + """;
            }

            QTabBar::tab {
                background-color: """ + COLOR_HACKER_BLACK + """;
            }

            QTabBar::tab:hover {
                background-color: """ + COLOR_LIGHT_GRAY + """;
            }

            QTabBar::tab:selected {
                background-color: """ + COLOR_LIGHT_GRAY2 + """;
            }
        """

    _THEME_NAVY = """
            QProgressBar {
                border: 1px solid gray;
                border-radius: 2px;
                background-color: white;
                text-align: center;
            }

            QProgressBar::chunk {
                border-radius: 2px;
                background-color: """ + COLOR_GREEN + """;
            }

            QWidget {
                background-color: """ + COLOR_NAVY_BLUE + """;
                color: white;
            }

            QPushButton {
                background-color: white;
                color: black;
            }

            QPushButton:hover {
                background-color: """ + COLOR_LIGHT_GRAY + """;
            }

            QComboBox, QAbstractSpinBox, QSpinBox {
                background-color: white;
                color: black;
            }

            QComboBox:disabled,
            QAbstractSpinBox:disabled,
            QSpinBox:disabled,
            QAbstractButton:disabled {
                color: gray;
            }

            QLineEdit, QAbstractScrollArea {
                background-color: white;
                color: black;
                border: 1px solid gray;
                margin: 3px;
                border-radius: 2px;
            }

            QMenu::item:selected {
                background-color: """ + COLOR_LIGHT_GRAY + """;
            }

            QTabWidget {
                border: 3px solid white;
            }

            QCheckBox::indicator {
                border: 1px solid white;
                background-color: """ + COLOR_NAVY_BLUE + """;
            }

            QCheckBox::indicator:checked {
                image: url(""" + ICON_PATH + '/checkmark_white.png' + """);
            }

            .QSlider {
                min-width: 100px;
                max-width: 100px;
            }

            QComboBox {
                selection-background-color: """ + COLOR_NAVY_BLUE + """;
                selection-color: white;
                color: black;
            }

            QComboBox, QAbstractItemView {
                color: black;
                background-color: white;
            }

            .QSlider::groove:vertical {
                border: 1px solid white;
                width: 10px;
            }

            .QSlider::handle:vertical {
                background: """ + COLOR_GREEN + """;
                border: 5px solid #B5E61D;
                height: 5px;
                border-radius: 30px;
            }

            QTabBar::tab {
                background-color: """ + COLOR_NAVY_LIGHT_BLUE + """;
            }

            QTabBar::tab:hover {
                background-color: """ + COLOR_LIGHT_GRAY + """;
            }

            QTabBar::tab:selected {
                background-color: """ + COLOR_LIGHT_GRAY2 + """;
            }

            QHeaderView {
                background-color: """ + COLOR_NAVY_LIGHT_BLUE + """;
                border-color: """ + COLOR_NAVY_LIGHT_BLUE + """;
                border-width: 0px;
                color: white;
            }

        """

    _THEMES = {
        'Default': _THEME_DEFAULT,
        'Navy blue': _THEME_NAVY,
        'Hacker': _THEME_HACKER,
        }

    @staticmethod
    def set_background_color(obj, red, green, blue):
        obj.setStyleSheet('background-color: rgb(%s, %s, %s)' %
                          (red, green, blue))

    @staticmethod
    def select_theme(theme):
        return UiUtils._THEMES[theme]

    @staticmethod
    def progressbar_stylesheet(color):

        return """
            QProgressBar {
                border: 1px solid gray;
                border-radius: 2px;
                text-align: center;
            }

            QProgressBar::chunk {
                border-radius: 2px;
                background-color: """ + color + """;
            }
            """



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#     ||          ____  _ __
#  +------+      / __ )(_) /_______________ _____  ___
#  | 0xBC |     / __  / / __/ ___/ ___/ __ `/_  / / _ \
#  +------+    / /_/ / / /_/ /__/ /  / /_/ / / /_/  __/
#   ||  ||    /_____/_/\__/\___/_/   \__,_/ /___/\___/
#
#  Copyright (C) 2013-2023 Bitcraze AB
#
#  Crazyflie Nano Quadcopter Client
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License along with
#  this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""
Shows all the parameters available in the Crazyflie and also gives the ability
to edit them.
"""

from PyQt6 import QtWidgets, uic
from PyQt6.QtCore import Qt, pyqtSignal

import cfclient
from cfclient.ui.tab_toolbox import TabToolbox

__author__ = 'Bitcraze AB'
__all__ = ['LogBlockDebugTab']

logblock_tab_class = uic.loadUiType(cfclient.module_path + "/ui/tabs/logBlockDebugTab.ui")[0]


class LogBlockDebugTab(TabToolbox, logblock_tab_class):
    """
    Used to show debug-information about log status.
    """

    _blocks_updated_signal = pyqtSignal(object, bool)
    _disconnected_signal = pyqtSignal(str)

    def __init__(self, helper):
        super(LogBlockDebugTab, self).__init__(helper, 'Log Blocks Debugging')
        self.setupUi(self)

        self._helper.cf.log.block_added_cb.add_callback(self._block_added)
        self._disconnected_signal.connect(self._disconnected)
        self._helper.cf.disconnected.add_callback(
            self._disconnected_signal.emit)
        self._blocks_updated_signal.connect(self._update_tree)

        self._block_tree.setHeaderLabels(
            ['Id', 'Name', 'Period (ms)', 'Added', 'Started', 'Error',
             'Contents'])
        self._block_tree.sortItems(0, Qt.SortOrder.AscendingOrder)

    def _block_added(self, block):
        """Callback when a new logblock has been created"""
        block.added_cb.add_callback(self._blocks_updated_signal.emit)
        block.started_cb.add_callback(self._blocks_updated_signal.emit)

    def _update_tree(self, conf, value):
        """Update the block tree"""
        self._block_tree.clear()
        for block in self._helper.cf.log.log_blocks:
            item = QtWidgets.QTreeWidgetItem()
            item.setFlags(Qt.ItemFlag.ItemIsEnabled |
                          Qt.ItemFlag.ItemIsSelectable)
            item.setData(0, Qt.ItemDataRole.DisplayRole, block.id)
            item.setData(1, Qt.ItemDataRole.EditRole, block.name)
            item.setData(2, Qt.ItemDataRole.DisplayRole, (block.period_in_ms))
            item.setData(3, Qt.ItemDataRole.DisplayRole, block.added)
            item.setData(4, Qt.ItemDataRole.EditRole, block.started)
            item.setData(5, Qt.ItemDataRole.EditRole, block.err_no)
            for var in block.variables:
                subItem = QtWidgets.QTreeWidgetItem()
                subItem.setFlags(Qt.ItemFlag.ItemIsEnabled |
                                 Qt.ItemFlag.ItemIsSelectable)
                subItem.setData(6, Qt.ItemDataRole.EditRole, var.name)
                item.addChild(subItem)

            self._block_tree.addTopLevelItem(item)
            self._block_tree.expandItem(item)

    def _disconnected(self, link_uri):
        """Callback when the Crazyflie is disconnected"""
        self._block_tree.clear()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#     ||          ____  _ __
#  +------+      / __ )(_) /_______________ _____  ___
#  | 0xBC |     / __  / / __/ ___/ ___/ __ `/_  / / _ \
#  +------+    / /_/ / / /_/ /__/ /  / /_/ / / /_/  __/
#   ||  ||    /_____/_/\__/\___/_/   \__,_/ /___/\___/
#
#  Copyright (C) 2014 Bitcraze AB
#
#  Crazyflie Nano Quadcopter Client
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA  02110-1301, USA.

"""
Find all the available input readers and try to import them.

To create a new input device reader drop a .py file into this
directory and it will be picked up automatically.
"""

import logging
from ..inputreaderinterface import InputReaderInterface

__author__ = 'Bitcraze AB'
__all__ = ['InputDevice']

logger = logging.getLogger(__name__)


# Forces py2exe to include the input readers in the windows build
try:
    from . import pysdl2  # noqa
    from . import linuxjsdev  # noqa
except Exception:
    pass

# Statically listing the available input readers
input_readers = ["linuxjsdev",
                 "pysdl2"]

logger.info("Input readers: {}".format(input_readers))

initialized_readers = []
available_devices = []

for reader in input_readers:
    try:
        module = __import__(reader, globals(), locals(), [reader], 1)
        main_name = getattr(module, "MODULE_MAIN")
        initialized_readers.append(getattr(module, main_name)())
        logger.info("Successfully initialized [{}]".format(reader))
    except Exception as e:
        logger.info("Could not initialize [{}]: {}".format(reader, e))


def devices():
    # Todo: Support rescanning and adding/removing devices
    if len(available_devices) == 0:
        for r in initialized_readers:
            devs = r.devices()
            for dev in devs:
                available_devices.append(InputDevice(dev["name"],
                                                     dev["id"],
                                                     r))
    return available_devices


class InputDevice(InputReaderInterface):

    def __init__(self, dev_name, dev_id, dev_reader):
        super(InputDevice, self).__init__(dev_name, dev_id, dev_reader)

        # All devices supports mapping (and can be configured)
        self.supports_mapping = True

        # Limit roll/pitch/yaw/thrust for all devices
        self.limit_rp = True
        self.limit_thrust = True
        self.limit_yaw = True
        self.db = 0.

    def open(self):
        # TODO: Reset data?
        self._reader.open(self.id)

    def close(self):
        self._reader.close(self.id)

    def set_dead_band(self, db):
        self.db = db

    def read(self, include_raw=False):
        [axis, buttons] = self._reader.read(self.id)

        # To support split axis we need to zero all the axis
        self.data.reset_axes()

        i = 0
        for a in axis:
            index = "Input.AXIS-%d" % i
            try:
                if self.input_map[index]["type"] == "Input.AXIS":
                    key = self.input_map[index]["key"]
                    axisvalue = a + self.input_map[index]["offset"]
                    axisvalue = axisvalue / self.input_map[index]["scale"]
                    self.data.set(key, axisvalue + self.data.get(key))
            except (KeyError, TypeError):
                pass
            i += 1

        # Workaround for fixing issues during mapping (remapping buttons while
        # they are pressed.
        self.data.reset_buttons()

        i = 0
        for b in buttons:
            index = "Input.BUTTON-%d" % i
            try:
                if self.input_map[index]["type"] == "Input.BUTTON":
                    key = self.input_map[index]["key"]
                    self.data.set(key, True if b == 1 else False)
            except (KeyError, TypeError):
                # Button not mapped, ignore..
                pass
            i += 1

        self.data.roll = InputDevice.deadband(self.data.roll, self.db)
        self.data.pitch = InputDevice.deadband(self.data.pitch, self.db)

        if self.limit_rp:
            [self.data.roll, self.data.pitch] = self._scale_rp(self.data.roll,
                                                               self.data.pitch)
        if self.limit_thrust:
            self.data.thrust = self._limit_thrust(self.data.thrust,
                                                  self.data.assistedControl,
                                                  self.data.estop)
        if self.limit_yaw:
            self.data.yaw = self._scale_and_deadband_yaw(self.data.yaw)

        if include_raw:
            return [axis, buttons, self.data]
        else:
            return self.data

    @staticmethod
    def deadband(value, threshold):
        if abs(value) < threshold:
            value = 0
        elif value > 0:
            value -= threshold
        elif value < 0:
            value += threshold
        return value / (1 - threshold)
